implementation module Data.Word where

import PreludeInternal
from StdString import class toString (..),instance toString Int,instance toString Double
from StdOverloaded import class toInt (..),class toReal(..)
from StdInt import instance toInt Double,bitxor,bitor,IF_INT_64_OR_32
from StdReal import entier,instance toReal Int
import StdArray

:: Word8:==Int
:: Word16:==Int
:: Word32:==Int

shiftr -: !Int !Int -> Int;
shiftr a b = `code inline {
	shiftr%
}

ushiftr -: !Int !Int -> Int;
ushiftr a b = `code inline {
	shiftrU
}

negate_int :: !Int -> Int
negate_int a = `code {
		negI
	}

and0xff :: !Int -> Int
and0xff i = `code inline {
	pushI 255
	and%
 }

and0xffff :: !Int -> Int
and0xffff i = `code inline {
	pushI 65535
	and%
 }

instance Eq Word8 where
	(==) :: !Word8 -> !Word8 -> Bool
	a == b = `code inline {
				eqI
		}
	(/=) :: !Word8 -> !Word8 -> Bool
	a /= b = `code inline {
				eqI
				notB
		}

instance Eq Word16 where
	(==) :: !Word16 -> !Word16 -> Bool
	a == b = `code inline {
				eqI
		}
	(/=) :: !Word16 -> !Word16 -> Bool
	a /= b = `code inline {
				eqI
				notB
		}

instance Eq Word32 where
	(==) :: !Word32 -> !Word32 -> Bool
	a == b = `code inline {
				eqI
		}
	(/=) :: !Word32 -> !Word32 -> Bool
	a /= b = `code inline {
				eqI
				notB
		}

instance Ord Word8 where
	compare :: !Word8 -> !Word8 -> Ordering
	compare a b
		= if a==b
		  then EQ
		  else  if a<b
				then LT
				else GT

	(<=) infix 4 :: !Word8 -> !Word8 -> Bool
	x <= y = `code inline {
				gtU
				notB
			}

	(<) infix  4 :: !Word8 -> !Word8 -> Bool
	x < y  = `code inline {
				ltU
			}

	(>=) infix 4 :: !Word8 -> !Word8 -> Bool
	x >= y = `code inline {
				ltU
				notB
			}

	(>) infix  4 :: !Word8 -> !Word8 -> Bool
	x > y  = `code inline {
				gtU
			} 

	min :: !Word8 -> !Word8 -> Word8
	min x y  = `code inline {
				.newlocallabel _WORD8_min
				push_b 1
				push_b 1
				ltU
				jmp_false _WORD8_min
				update_b 0 1
			:_WORD8_min
				pop_b 1
			}

	max :: !Word8 -> !Word8 -> Word8
	max x y = `code inline {
				.newlocallabel _WORD8_max
				push_b 1
				push_b 1
				gtU
				jmp_false _WORD8_max
				update_b 0 1
			:_WORD8_max
				pop_b 1
			}

instance Ord Word32 where
	compare :: !Word32 -> !Word32 -> Ordering
	compare a b
		= if a==b
		  then EQ
		  else  if a<b
				then LT
				else GT

	(<=) infix 4 :: !Word32 -> !Word32 -> Bool
	x <= y = `code inline {
				gtU
				notB
			}

	(<) infix  4 :: !Word32 -> !Word32 -> Bool
	x < y  = `code inline {
				ltU
			}

	(>=) infix 4 :: !Word32 -> !Word32 -> Bool
	x >= y = `code inline {
				ltU
				notB
			}

	(>) infix  4 :: !Word32 -> !Word32 -> Bool
	x > y  = `code inline {
				gtU
			} 

	min :: !Word32 -> !Word32 -> Word32
	min x y  = `code inline {
				.newlocallabel _WORD32_min
				push_b 1
				push_b 1
				ltU
				jmp_false _WORD32_min
				update_b 0 1
			:_WORD32_min
				pop_b 1
			}

	max :: !Word32 -> !Word32 -> Word32
	max x y = `code inline {
				.newlocallabel _WORD32_max
				push_b 1
				push_b 1
				gtU
				jmp_false _WORD32_max
				update_b 0 1
			:_WORD32_max
				pop_b 1
			}

instance Num Word8 where
	(+) :: !Word8 -> !Word8 -> Word8
	a + b
		= `code inline {
			addI
			pushI 255
			and%
		}
	(-) :: !Word8 -> !Word8 -> Word8
	a - b
		= `code inline {
			subI
			pushI 255
			and%
		}
	(*) :: !Word8 -> !Word8 -> Word8
	a * b
		= `code inline {
			mulI
			pushI 255
			and%
		}
	abs :: !Word8 -> Word8
	abs a
		= a
	signum :: !Word8 -> Word8
	signum a
		= ushiftr (a bitor (negate_int a)) (IF_INT_64_OR_32 63 31)
	negate :: !Word8 -> Word8
	negate a
		= `code inline {
			negI
			pushI 255
			and%
		}
	fromInteger :: !Integer -> Word8
	fromInteger {integer_s,integer_a}
		| size integer_a==`0
			= and0xff integer_s;
			= and0xff ((integer_a~[0] bitxor integer_s) - integer_s);

instance Num Word16 where
	(+) :: !Word16 -> !Word16 -> Word16
	a + b
		= `code inline {
			addI
			pushI 65535
			and%
		}
	(-) :: !Word16 -> !Word16 -> Word16
	a - b
		= `code inline {
			subI
			pushI 65535
			and%
		}
	(*) :: !Word16 -> !Word16 -> Word16
	a * b
		= `code inline {
			mulI
			pushI 65535
			and%
		}
	abs :: !Word16 -> Word16
	abs a
		= a
	signum :: !Word16 -> Word16
	signum a
		= ushiftr (a bitor (negate_int a)) (IF_INT_64_OR_32 63 31)
	negate :: !Word16 -> Word16
	negate a
		= `code inline {
			negI
			pushI 65535
			and%
		}
	fromInteger :: !Integer -> Word16
	fromInteger {integer_s,integer_a}
		| size integer_a==`0
			= and0xffff integer_s;
			= and0xffff ((integer_a~[0] bitxor integer_s) - integer_s);

instance Num Word32 where
	(+) :: !Word32 -> !Word32 -> Word32
	a + b
		= `code inline {
			addI
		}
	(-) :: !Word32 -> !Word32 -> Word32
	a - b
		= `code inline {
			subI
		}
	(*) :: !Word32 -> !Word32 -> Word32
	a * b
		= `code inline {
			mulI
		}
	abs :: !Word32 -> Word32
	abs a
		= a
	signum :: !Word32 -> Word32
	signum a
		= ushiftr (a bitor (negate_int a)) (IF_INT_64_OR_32 63 31)
	negate :: !Word32 -> Word32
	negate a
		= `code inline {
			negI
		}
	fromInteger :: !Integer -> Word32
	fromInteger {integer_s,integer_a}
		| size integer_a==`0
			= integer_s;
			= (integer_a~[0] bitxor integer_s) - integer_s;

int_to_integer :: !Int -> Integer
int_to_integer i = `code inline {
	pushI 0
	create_array_ INT 0 1
}

instance Integral Word8 where
	toInteger :: !Word8 -> Integer
	toInteger i
		= int_to_integer i

	quot :: !Word8 -> !Word8 -> Word8
	quot a b = `code inline {
		divU
	}

	rem :: !Word8 -> !Word8 -> Word8
	rem a b = `code inline {
		pushI 0
		divLU
		updatepop_b 0 1
	}

	div :: !Word8 -> !Word8 -> Word8
	div a b = `code inline {
		divU
	}

	mod :: !Word8 -> !Word8 -> Word8
	mod a b = `code inline {
		pushI 0
		divLU
		updatepop_b 0 1
	}

	quotRem :: !Word8 -> !Word8 -> (!Word8,!Word8)
	quotRem a b = `code inline {
		pushI 0
		divLU
		push_b 1
		update_b 1 2
		updatepop_b 0 1
	}

	divMod :: !Word8 -> !Word8 -> (!Word8,!Word8)
	divMod a b = `code inline {
		pushI 0
		divLU
		push_b 1
		update_b 1 2
		updatepop_b 0 1
	}

instance Integral Word32 where
	toInteger :: !Word32 -> Integer
	toInteger i
		| i>=`0
			= int_to_integer i
			= {integer_s=`0,integer_a={i}}

	quot :: !Word32 -> !Word32 -> Word32
	quot a b = `code inline {
		divU
	}

	rem :: !Word32 -> !Word32 -> Word32
	rem a b = `code inline {
		pushI 0
		divLU
		updatepop_b 0 1
	}

	div :: !Word32 -> !Word32 -> Word32
	div a b = `code inline {
		divU
	}

	mod :: !Word32 -> !Word32 -> Word32
	mod a b = `code inline {
		pushI 0
		divLU
		updatepop_b 0 1
	}

	quotRem :: !Word32 -> !Word32 -> (!Word32,!Word32)
	quotRem a b = `code inline {
		pushI 0
		divLU
		push_b 1
		update_b 1 2
		updatepop_b 0 1
	}

	divMod :: !Word32 -> !Word32 -> (!Word32,!Word32)
	divMod a b = `code inline {
		pushI 0
		divLU
		push_b 1
		update_b 1 2
		updatepop_b 0 1
	}

instance Enum Word8 where
	toEnum :: !Int -> Word8
	toEnum i = i

	fromEnum :: !Word8 -> Int
	fromEnum i = i

instance Enum Word32 where
	toEnum :: !Int -> Word32
	toEnum i = i

	fromEnum :: !Word32 -> Int
	fromEnum i = i

instance Show Word8 where
	show :: !Word8 -> [Char]
	show i = string_to_char_list (toString i)
	showsPrec _ i = string_to_char_list_append (toString i)

instance Show Word16 where
	show :: !Word16 -> [Char]
	show i = string_to_char_list (toString i)
	showsPrec _ i = string_to_char_list_append (toString i)

instance Show Word32 where
	show :: !Word32 -> [Char]
	--to do: fix if > max int
	show i = string_to_char_list (toString i)
	showsPrec _ i = string_to_char_list_append (toString i)

instance Real Word8 where
	toRational :: !Word8 -> Rational
	toRational i
		= {integer_s=i,integer_a={}} :% 1

instance Real Word32 where
	toRational :: !Word32 -> Rational
	toRational i
		| i>=`0
			= {integer_s=i,integer_a={}} :% 1
			= {integer_s=0,integer_a={i}} :% 1
