implementation module Foreign.C.String

import Prelude

import Foreign.Ptr,Foreign.C.Types,Foreign.Marshal.Alloc

:: CString :== Ptr CChar
:: CStringLen :== (Ptr CChar,Int)

withCString :: ![Char] !(CString -> IO a) -> IO a
withCString char_list f
	#! (n_chars,char_list) = eval_and_length char_list
	= allocaBytes (n_chars+1) (copy_chars_and_call_f char_list)
where
	copy_chars_and_call_f char_list p
		#! p = copy_chars_and_0 0 char_list p
		= f p;

	copy_chars_and_0 :: !Int ![!Char!] !CString -> CString
	copy_chars_and_0 n [!c:cs!] p
		#! p=store_char_p p n c
		= copy_chars_and_0 (n+1) cs p;
	copy_chars_and_0 n [!!] p
		= store_char_p p n '\0'

withCStringLen :: ![Char] !(CStringLen -> IO a) -> IO a
withCStringLen char_list f
	#! (n_chars,char_list) = eval_and_length char_list
	= allocaBytes n_chars (copy_chars_and_call_f char_list n_chars)
where
	copy_chars_and_call_f char_list n_chars p
		#! p = copy_chars 0 char_list p
		= f (p,n_chars);

	copy_chars :: !Int ![!Char!] !CString -> CString
	copy_chars n [!c:cs!] p
		#! p=store_char_p p n c
		= copy_chars (n+1) cs p;
	copy_chars n [!!] p
		= p

withCAStringLen :: ![Char] !(CStringLen -> IO a) -> IO a
withCAStringLen char_list f = withCStringLen char_list f

store_char_p :: !CString !Int !Char -> CString
store_char_p p i c = code {
		updatepop_b 0 2
	|	mov    (%esp),%ecx
		instruction	139
		instruction 12
		instruction 36
	|	movb	%cl,(%eax,%ebx,1)
		instruction 136
		instruction 12
		instruction 24
 }

eval_and_length :: ![Char] -> (!Int,![!Char!])
eval_and_length cs
	#! n=eval_and_length cs 0;
	= cast n cs
where
	eval_and_length :: ![Char] !Int -> Int
	eval_and_length [c:cl] l
		= seq c (eval_and_length cl (l+1))
	eval_and_length [] l
		= l
	
	cast :: !Int ![Char] -> (!Int,![!Char!])
	cast l cs = code {
		pop_a 0
	}

peekCString :: !CString -> IO [Char]
peekCString p
	#! chars = c_string_to_char_list p
	= IO (\ w -> (chars,w))
where
	c_string_to_char_list p
		# (c,p) = next_c_string_char p
		| c=='\0'
			= []
			#! cs = c_string_to_char_list p
			= [c : cs]
	
peekCStringLen :: !CStringLen -> IO [Char]
peekCStringLen (p,n)
	#! chars = c_string_to_char_list n p
	= IO (\ w -> (chars,w))
where
	c_string_to_char_list :: !Int !CString -> [Char]
	c_string_to_char_list n p
		| n>0
			# (c,p) = next_c_string_char p
			#! cs = c_string_to_char_list (n-1) p
			= [c : cs]
			= []

peekCAStringLen :: !CStringLen -> IO [Char]
peekCAStringLen s = peekCStringLen s

next_c_string_char :: !CString -> (!Char,!CString)
next_c_string_char p = code inline {
	push_b 0
	load_ui8 0
	push_b 1
	pushI 1
	addI
	update_b 0 2
	pop_b 1
 }

