implementation module Foreign.Marshal.Array where

import Prelude
import Data.Bits
import Foreign.Ptr,Foreign.Storable,Foreign.Marshal.Alloc

import _SystemStrictLists

allocaArray :: (Storable a) => !Int -> !(Ptr a -> IO b) -> IO b 
allocaArray n f
	| n>=0
		#! a=a
		#! (a,f) = (a,f) :: (a,(Ptr a) -> IO b);
		= allocaBytes (n*sizeOf a) f
 where	
	a :: a
	a = `code inline {
		buildh _Nil 0
	 }

pokeArray :: (Storable a) => !(Ptr a) -> ![a] -> IO ()
pokeArray p vs
	= poke_array vs p `0
  where
	poke_array :: ![a] -> !(Ptr a) -> !Int -> IO () | Storable a
	poke_array (v:vs) p i
		= do pokeElemOff p i v
			 poke_array vs p (i+`1)
	poke_array [] p i
		= return ()

peekArray :: (Storable a) => !Int -> !Ptr a -> IO [a]
peekArray array_size p
	= peek_array `0 array_size p
  where
  	peek_array :: !Int -> !Int -> !(Ptr a) -> IO [a] | Storable a
	peek_array i s p
		| i<s
			= do
				e <- peekElemOff p i
				l <- peek_array (i+`1) s p
				return (e:l)
			= return []

withArray :: (Storable a) => ![a] -> !(Ptr a -> IO b) -> IO b
withArray l f
	# (n,l) = eval_and_length l
	#! (a,f) = (a,f) :: (a,(Ptr a) -> IO b);
	= allocaBytes (n*sizeOf a) (copy_array_and_call_f l f)
 where
	a :: a
	a = `code inline {
		buildh _Nil 0
	 }

	copy_array_and_call_f l f p
		= do copy_array l p `0
			 f p

withArrayLen :: (Storable a) => ![a] -> !(Int -> Ptr a -> IO b) -> IO b
withArrayLen l f
	# (n,l) = eval_and_length l
	#! (a,f) = (a,f) :: (a,Int -> (Ptr a) -> IO b);
	= allocaBytes (n*sizeOf a) (copy_array_and_call_f l (f n))
 where
	a :: a
	a = `code inline {
		buildh _Nil 0
	 }

	copy_array_and_call_f l f p
		= do copy_array l p `0
			 f p

copy_array :: ![!a!] -> !(Ptr a) -> !Int -> IO (Ptr a) | Storable a
copy_array [!!] p n
	= return p
copy_array [|v;vs] p n
	= do pokeElemOff p n v
		 copy_array vs p (n+`1)

eval_and_length :: ![a] -> (!Int,![!a!])
eval_and_length cs
	#! n=eval_and_length cs 0;
	= cast n cs
where
	eval_and_length :: ![a] -> !Int -> Int
	eval_and_length (c:cl) l
		= seq c (eval_and_length cl (l+1))
	eval_and_length [] l
		= l
	
	cast :: !Int -> ![a] -> (!Int,![!a!])
	cast l cs = `code {
		pop_a 0
	}
