definition module Foreign.Storable where

from PreludeTypes import qualified ::IO,::(),class Num(..)
from Data.Word import qualified::Word8,::Word32
from Data.Int import qualified ::Int32
from Foreign.Ptr import qualified ::Ptr,plusPtr

class Storable a where
	peek :: (Foreign.Ptr.Ptr a) -> PreludeTypes.IO a
	peek ptr = peekElemOff ptr `0
	poke :: (Foreign.Ptr.Ptr a) -> a -> PreludeTypes.IO PreludeTypes.`Unit
	poke ptr v = pokeElemOff ptr `0 v
	peekElemOff :: (Foreign.Ptr.Ptr a) -> Int -> PreludeTypes.IO a
	peekElemOff ptr off = peekByteOff ptr (off PreludeTypes.* sizeOf (`Ptr_a_to_a ptr))
	pokeElemOff :: (Foreign.Ptr.Ptr a) -> Int -> a -> PreludeTypes.IO PreludeTypes.`Unit
	pokeElemOff ptr off val = pokeByteOff ptr (off PreludeTypes.* sizeOf val) val
	peekByteOff :: (Foreign.Ptr.Ptr a) -> Int -> PreludeTypes.IO a
	peekByteOff ptr off = peek (Foreign.Ptr.plusPtr ptr off)
	pokeByteOff :: (Foreign.Ptr.Ptr a) -> Int -> a -> PreludeTypes.IO PreludeTypes.`Unit
	pokeByteOff ptr off v = poke (Foreign.Ptr.plusPtr ptr off) v
	sizeOf :: a->Int
	alignment :: a->Int

`Ptr_a_to_a :: (Foreign.Ptr.Ptr a) -> a

instance Storable Int where
	peek :: !(Foreign.Ptr.Ptr Int) -> PreludeTypes.IO Int
	poke :: !(Foreign.Ptr.Ptr Int) -> !Int -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr Int) -> !Int -> PreludeTypes.IO Int
	pokeElemOff :: !(Foreign.Ptr.Ptr Int) -> !Int -> !Int -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr Int) -> !Int -> PreludeTypes.IO Int
	pokeByteOff :: !(Foreign.Ptr.Ptr Int) -> !Int -> !Int -> PreludeTypes.IO PreludeTypes.`Unit

instance Storable Data.Int.Int32 where
	peek :: !(Foreign.Ptr.Ptr Data.Int.Int32) -> PreludeTypes.IO Data.Int.Int32
	poke :: !(Foreign.Ptr.Ptr Data.Int.Int32) -> !Data.Int.Int32 -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr Data.Int.Int32) -> !Int -> PreludeTypes.IO Data.Int.Int32
	pokeElemOff :: !(Foreign.Ptr.Ptr Data.Int.Int32) -> !Int -> !Data.Int.Int32 -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr Data.Int.Int32) -> !Int -> PreludeTypes.IO Data.Int.Int32
	pokeByteOff :: !(Foreign.Ptr.Ptr Data.Int.Int32) -> !Int -> !Data.Int.Int32 -> PreludeTypes.IO PreludeTypes.`Unit

instance Storable Data.Word.Word8 where
	peek :: !(Foreign.Ptr.Ptr Data.Word.Word8) -> PreludeTypes.IO Data.Word.Word8
	poke :: !(Foreign.Ptr.Ptr Data.Word.Word8) -> !Data.Word.Word8 -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr Data.Word.Word8) -> !Int -> PreludeTypes.IO Data.Word.Word8
	pokeElemOff :: !(Foreign.Ptr.Ptr Data.Word.Word8) -> !Int -> !Data.Word.Word8 -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr Data.Word.Word8) -> !Int -> PreludeTypes.IO Data.Word.Word8
	pokeByteOff :: !(Foreign.Ptr.Ptr Data.Word.Word8) -> !Int -> !Data.Word.Word8 -> PreludeTypes.IO PreludeTypes.`Unit

instance Storable Data.Word.Word32 where
	peek :: !(Foreign.Ptr.Ptr Data.Word.Word32) -> PreludeTypes.IO Data.Word.Word32
	poke :: !(Foreign.Ptr.Ptr Data.Word.Word32) -> !Data.Word.Word32 -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr Data.Word.Word32) -> !Int -> PreludeTypes.IO Data.Word.Word32
	pokeElemOff :: !(Foreign.Ptr.Ptr Data.Word.Word32) -> !Int -> !Data.Word.Word32 -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr Data.Word.Word32) -> !Int -> PreludeTypes.IO Data.Word.Word32
	pokeByteOff :: !(Foreign.Ptr.Ptr Data.Word.Word32) -> !Int -> !Data.Word.Word32 -> PreludeTypes.IO PreludeTypes.`Unit

instance Storable Char where
	peek :: !(Foreign.Ptr.Ptr Char) -> PreludeTypes.IO Char
	poke :: !(Foreign.Ptr.Ptr Char) -> !Char -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr Char) -> !Int -> PreludeTypes.IO Char
	pokeElemOff :: !(Foreign.Ptr.Ptr Char) -> !Int -> !Char -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr Char) -> !Int -> PreludeTypes.IO Char
	pokeByteOff :: !(Foreign.Ptr.Ptr Char) -> !Int -> !Char -> PreludeTypes.IO PreludeTypes.`Unit

instance Storable Float where
	peek :: !(Foreign.Ptr.Ptr Float) -> PreludeTypes.IO Float
	poke :: !(Foreign.Ptr.Ptr Float) -> !Float -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr Float) -> !Int -> PreludeTypes.IO Float
	pokeElemOff :: !(Foreign.Ptr.Ptr Float) -> !Int -> !Float -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr Float) -> !Int -> PreludeTypes.IO Float
	pokeByteOff :: !(Foreign.Ptr.Ptr Float) -> !Int -> !Float -> PreludeTypes.IO PreludeTypes.`Unit

instance Storable Double where
	peek :: !(Foreign.Ptr.Ptr Double) -> PreludeTypes.IO Double
	poke :: !(Foreign.Ptr.Ptr Double) -> !Double -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr Double) -> !Int -> PreludeTypes.IO Double
	pokeElemOff :: !(Foreign.Ptr.Ptr Double) -> !Int -> !Double -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr Double) -> !Int -> PreludeTypes.IO Double
	pokeByteOff :: !(Foreign.Ptr.Ptr Double) -> !Int -> !Double -> PreludeTypes.IO PreludeTypes.`Unit

instance Storable (Foreign.Ptr.Ptr a) where
	peek :: !(Foreign.Ptr.Ptr (Foreign.Ptr.Ptr a)) -> PreludeTypes.IO (Foreign.Ptr.Ptr a)
	poke :: !(Foreign.Ptr.Ptr (Foreign.Ptr.Ptr a)) -> !(Foreign.Ptr.Ptr a) -> PreludeTypes.IO PreludeTypes.`Unit
	peekElemOff :: !(Foreign.Ptr.Ptr (Foreign.Ptr.Ptr a)) -> !Int -> PreludeTypes.IO (Foreign.Ptr.Ptr a)
	pokeElemOff :: !(Foreign.Ptr.Ptr (Foreign.Ptr.Ptr a)) -> !Int -> !(Foreign.Ptr.Ptr a) -> PreludeTypes.IO PreludeTypes.`Unit
	peekByteOff :: !(Foreign.Ptr.Ptr (Foreign.Ptr.Ptr a)) -> !Int -> PreludeTypes.IO (Foreign.Ptr.Ptr a)
	pokeByteOff :: !(Foreign.Ptr.Ptr (Foreign.Ptr.Ptr a)) -> !Int -> !(Foreign.Ptr.Ptr a) -> PreludeTypes.IO PreludeTypes.`Unit
