implementation module Foreign.Storable

import Prelude
from PreludeTypes import qualified ::()
import Data.Word,Data.Int
import Foreign.Ptr
from Data.Int import qualified ::Int32
from Data.Word import qualified ::Word8,::Word32
from Foreign.Ptr import qualified ::Ptr

class Storable a where
	peek :: (Ptr a) -> IO a
	poke :: (Ptr a) a -> IO PreludeTypes.()
	peekElemOff :: (Ptr a) Int -> IO a
	pokeElemOff :: (Ptr a) Int a -> IO PreludeTypes.()
	peekByteOff :: (Ptr a) Int -> IO a
	pokeByteOff :: (Ptr a) Int a -> IO PreludeTypes.()
	sizeOf :: a->Int
	alignment :: a->Int

	peek ptr = peekElemOff ptr 0
	poke ptr v = pokeElemOff ptr 0 v
	peekElemOff ptr off = peekByteOff ptr (off * sizeOf (`Ptr_a_to_a ptr))
	pokeElemOff ptr off val = pokeByteOff ptr (off * sizeOf val) val
	peekByteOff ptr off = peek (plusPtr ptr off)
	pokeByteOff ptr off v = poke (plusPtr ptr off) v

`Ptr_a_to_a :: (Ptr a) -> a
`Ptr_a_to_a  p => `Ptr_a_to_a p

with_world f :== IO (\w->f w)

instance Storable Int where
	peek :: !(Ptr Int) -> IO Int
	peek p = with_world (w`peek_int p)
	poke :: !(Ptr Int) !Int -> IO PreludeTypes.()
	poke p v = with_world (w`poke_int p v)
	peekElemOff :: !(Ptr Int) !Int -> IO Int
	peekElemOff p i = with_world (w`peek_elem_off_int p i)
	pokeElemOff :: !(Ptr Int) !Int !Int -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_elem_off_int p i v)
	peekByteOff :: !(Ptr Int) !Int -> IO Int
	peekByteOff p i = with_world (w`peek_byte_off_int p i)
	pokeByteOff :: !(Ptr Int) !Int !Int -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_byte_off_int p i v)
	sizeOf _ = 4
	alignment _ = 4

instance Storable Data.Int.Int32 where
	peek :: !(Ptr Int32) -> IO Int32
	peek p = with_world (w`peek_int32 p)
	poke :: !(Ptr Data.Int.Int32) !Data.Int.Int32 -> IO PreludeTypes.()
	poke p v = with_world (w`poke_int32 p v)
	peekElemOff :: !(Ptr Data.Int.Int32) !Int -> IO Data.Int.Int32
	peekElemOff p i = with_world (w`peek_elem_off_int32 p i)
	pokeElemOff :: !(Ptr Data.Int.Int32) !Int !Data.Int.Int32 -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_elem_off_int32 p i v)
	peekByteOff :: !(Ptr Data.Int.Int32) !Int -> IO Data.Int.Int32
	peekByteOff p i = with_world (w`peek_byte_off_int32 p i)
	pokeByteOff :: !(Ptr Data.Int.Int32) !Int !Data.Int.Int32 -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_byte_off_int32 p i v)
	sizeOf _ = 4
	alignment _ = 4

instance Storable Data.Word.Word8 where
	peek :: !(Ptr Word8) -> IO Word8
	peek p = with_world (w`peek_word8 p)
	poke :: !(Ptr Word8) !Word8 -> IO PreludeTypes.()
	poke p v = with_world (w`poke_word8 p v)
	peekElemOff :: !(Ptr Word8) !Int -> IO Word8
	peekElemOff p i = with_world (w`peek_off_word8 p i)
	pokeElemOff :: !(Ptr Word8) !Int !Word8 -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_off_word8 p i v)
	peekByteOff :: !(Ptr Word8) !Int -> IO Word8
	peekByteOff p i = with_world (w`peek_off_word8 p i)
	pokeByteOff :: !(Ptr Word8) !Int !Word8 -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_off_word8 p i v)
	sizeOf _ = 1
	alignment _ = 1

instance Storable Data.Word.Word32 where
	peek :: !(Ptr Word32) -> IO Word32
	peek p = with_world (w`peek_word32 p)
	poke :: !(Ptr Word32) !Word32 -> IO PreludeTypes.()
	poke p v = with_world (w`poke_word32 p v)
	peekElemOff :: !(Ptr Word32) !Int -> IO Word32
	peekElemOff p i = with_world (w`peek_elem_off_word32 p i)
	pokeElemOff :: !(Ptr Word32) !Int !Word32 -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_elem_off_word32 p i v)
	peekByteOff :: !(Ptr Word32) !Int -> IO Word32
	peekByteOff p i = with_world (w`peek_byte_off_word32 p i)
	pokeByteOff :: !(Ptr Word32) !Int !Word32 -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_byte_off_word32 p i v)
	sizeOf _ = 4
	alignment _ = 4

instance Storable Char where
	peek :: !(Ptr Char) -> IO Char
	peek p = with_world (w`peek_char p)
	poke :: !(Ptr Char) !Char -> IO PreludeTypes.()
	poke p v = with_world (w`poke_char p v)
	peekElemOff :: !(Ptr Char) !Int -> IO Char
	peekElemOff p i = with_world (w`peek_off_char p i)
	pokeElemOff :: !(Ptr Char) !Int !Char -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_off_char p i v)
	peekByteOff :: !(Ptr Char) !Int -> IO Char
	peekByteOff p i = with_world (w`peek_off_char p i)
	pokeByteOff :: !(Ptr Char) !Int !Char -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_off_char p i v)
	sizeOf _ = 1
	alignment _ = 1

instance Storable Float where
	peek :: !(Ptr Float) -> IO Float
	peek p = with_world (w`peek_float p)
	poke :: !(Ptr Float) !Float -> IO PreludeTypes.()
	poke p v = with_world (w`poke_float p v)
	peekElemOff :: !(Ptr Float) !Int -> IO Float
	peekElemOff p i = with_world (w`peek_elem_off_float p i)
	pokeElemOff :: !(Ptr Float) !Int !Float -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_elem_off_float p i v)
	peekByteOff :: !(Ptr Float) !Int -> IO Float
	peekByteOff p i = with_world (w`peek_byte_off_float p i)
	pokeByteOff :: !(Ptr Float) !Int !Float -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_byte_off_float p i v)
	sizeOf _ = 4
	alignment _ = 4

instance Storable Real where
	peek :: !(Ptr Real) -> IO Real
	peek p = with_world (w`peek_double p)
	poke :: !(Ptr Real) !Real -> IO PreludeTypes.()
	poke p v = with_world (w`poke_double p v)
	peekElemOff :: !(Ptr Real) !Int -> IO Real
	peekElemOff p i = with_world (w`peek_elem_off_double p i)
	pokeElemOff :: !(Ptr Real) !Int !Real -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_elem_off_double p i v)
	peekByteOff :: !(Ptr Real) !Int -> IO Real
	peekByteOff p i = with_world (w`peek_byte_off_double p i)
	pokeByteOff :: !(Ptr Real) !Int !Real -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_byte_off_double p i v)
	sizeOf _ = 8
	alignment _ = 4

instance Storable (Foreign.Ptr.Ptr a) where
	peek :: !(Ptr (Ptr a)) -> IO (Ptr a)
	peek p = with_world (w`peek_ptr p)
	poke :: !(Ptr (Ptr a)) !(Ptr a) -> IO PreludeTypes.()
	poke p v = with_world (w`poke_ptr p v)
	peekElemOff :: !(Ptr (Ptr a)) !Int -> IO (Ptr a)
	peekElemOff p i = with_world (w`peek_elem_off_ptr p i)
	pokeElemOff :: !(Ptr (Ptr a)) !Int !(Ptr a) -> IO PreludeTypes.()
	pokeElemOff p i v = with_world (w`poke_elem_off_ptr p i v)
	peekByteOff :: !(Ptr (Ptr a)) !Int -> IO (Ptr a)
	peekByteOff p i = with_world (w`peek_byte_off_ptr p i)
	pokeByteOff :: !(Ptr (Ptr a)) !Int !(Ptr a) -> IO PreludeTypes.()
	pokeByteOff p i v = with_world (w`poke_byte_off_ptr p i v)
	sizeOf _ = 4
	alignment _ = 4

w`peek_int :: !(Ptr Int) !*World -> (!Int,!*World);
w`peek_int p w = code {
	load_i 0
 }

w`peek_int32 :: !(Ptr Int32) !*World -> (!Int32,!*World);
w`peek_int32 p w = code {
	load_i 0
 }

w`peek_word8 :: !(Ptr Word8) !*World -> (!Word8,!*World);
w`peek_word8 p w = code {
	load_ui8 0
 }

w`peek_word32 :: !(Ptr Word32) !*World -> (!Word32,!*World);
w`peek_word32 p w = code {
	load_i 0
 }

w`peek_char :: !(Ptr Char) !*World -> (!Char,!*World);
w`peek_char p w = code {
	load_ui8 0
 }

w`peek_byte_off_int :: !(Ptr Int) !Int !*World -> (!Int,!*World);
w`peek_byte_off_int p i w = code {
	addI
	load_i 0
 }

w`peek_byte_off_int32 :: !(Ptr Int32) !Int !*World -> (!Int32,!*World);
w`peek_byte_off_int32 p i w = code {
	addI
	load_i 0
 }

w`peek_off_word8 :: !(Ptr Word8) !Int !*World -> (!Word8,!*World);
w`peek_off_word8 p i w = code {
	addI
	load_ui8 0
 }

w`peek_off_char :: !(Ptr Char) !Int !*World -> (!Char,!*World);
w`peek_off_char p i w = code {
	addI
	load_ui8 0
 }

w`peek_byte_off_word32 :: !(Ptr Word32) !Int !*World -> (!Word32,!*World);
w`peek_byte_off_word32 p i w = code {
	addI
	load_i 0
 }

w`peek_elem_off_int :: !(Ptr Int) !Int !*World -> (!Int,!*World);
w`peek_elem_off_int p i w = code {
	pushI 2
	push_b 2
	shiftl%
	addI
	updatepop_b 0 1
	load_i 0
 }

w`peek_elem_off_int32 :: !(Ptr Int32) !Int !*World -> (!Int32,!*World);
w`peek_elem_off_int32 p i w = code {
	pushI 2
	push_b 2
	shiftl%
	addI
	updatepop_b 0 1
	load_i 0
 }

w`peek_elem_off_word32 :: !(Ptr Word32) !Int !*World -> (!Word32,!*World);
w`peek_elem_off_word32 p i w = code {
	pushI 2
	push_b 2
	shiftl%
	addI
	updatepop_b 0 1
	load_i 0
 }

w`poke_int :: !(Ptr Int) !Int !*World -> (!PreludeTypes.(),!*World)
w`poke_int p v w = code {
	:poke_int_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_int_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_int32 :: !(Ptr Int32) !Int32 !*World -> (!PreludeTypes.(),!*World)
w`poke_int32 p v w = code {
	:poke_int32_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_int32_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_word8 :: !(Ptr Word8) !Word8 !*World -> (!PreludeTypes.(),!*World)
w`poke_word8 p v w = code {
	:poke_word8_0
	|	movb	%al,(%ebx)
	instruction 136
	instruction 3
	:poke_word8_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_word32 :: !(Ptr Word32) !Word32 !*World -> (!PreludeTypes.(),!*World)
w`poke_word32 p v w = code {
	:poke_word32_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_word32_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_char :: !(Ptr Char) !Char !*World -> (!PreludeTypes.(),!*World)
w`poke_char p v w = code {
	:poke_char_0
	|	movb	%al,(%ebx)
	instruction 136
	instruction 3
	:poke_char_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_off_word8 :: !(Ptr Word8) !Int !Word8 !*World -> (!PreludeTypes.(),!*World)
w`poke_off_word8 p i w = code {
	addI
	.o 1 2 ii
	:poke_elem_off_word8_0
	|	movb	%al,(%ebx)
	instruction 136
	instruction 3
	:poke_elem_off_word8_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_off_char :: !(Ptr Char) !Int !Char !*World -> (!PreludeTypes.(),!*World)
w`poke_off_char p i w = code {
	addI
	.o 1 2 ii
	:poke_elem_off_char_0
	|	movb	%al,(%ebx)
	instruction 136
	instruction 3
	:poke_elem_off_char_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_byte_off_int :: !(Ptr Int) !Int !Int !*World -> (!PreludeTypes.(),!*World)
w`poke_byte_off_int p i v w = code {
	addI
	.o 1 2 ii
	:poke_byte_off_int_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_byte_off_int_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_byte_off_int32 :: !(Ptr Int32) !Int !Int32 !*World -> (!PreludeTypes.(),!*World)
w`poke_byte_off_int32 p i v w = code {
	addI
	.o 1 2 ii
	:poke_byte_off_int32_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_byte_off_int32_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_byte_off_word32 :: !(Ptr Word32) !Int !Word32 !*World -> (!PreludeTypes.(),!*World)
w`poke_byte_off_word32 p i v w = code {
	addI
	.o 1 2 ii
	:poke_byte_off_word32_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_byte_off_word32_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_elem_off_int :: !(Ptr Int) !Int !Int !*World -> (!PreludeTypes.(),!*World)
w`poke_elem_off_int p i v w = code {
	pushI 2
	push_b 2
	shiftl%
	addI
	updatepop_b 0 1
	.o 1 2 ii
	:poke_elem_off_int_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_elem_off_int_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_elem_off_int32 :: !(Ptr Int32) !Int !Int32 !*World -> (!PreludeTypes.(),!*World)
w`poke_elem_off_int32 p i v w = code {
	pushI 2
	push_b 2
	shiftl%
	addI
	updatepop_b 0 1
	.o 1 2 ii
	:poke_elem_off_int32_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_elem_off_int32_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_elem_off_word32 :: !(Ptr Word32) !Int !Word32 !*World -> (!PreludeTypes.(),!*World)
w`poke_elem_off_word32 p i v w = code {
	pushI 2
	push_b 2
	shiftl%
	addI
	updatepop_b 0 1
	.o 1 2 ii
	:poke_elem_off_word32_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:poke_elem_off_word32_1
	pop_b 2
	buildh e_PreludeTypes_d`Unit 0
 }

w`peek_float :: !(Ptr Float) !*World -> (!Float,!*World);
w`peek_float p w = code {
	pushR 0.0
	update_b 1 2
	updatepop_b 0 1
	jmp peek_float_0
	:peek_float_0
	instruction 217
	instruction 0	| flds (%eax)
	instruction 221
	instruction 217 | fstp   %st(1)
 }

w`peek_byte_off_float :: !(Ptr Float) !Int !*World -> (!Float,!*World);
w`peek_byte_off_float p i w = code {
	pushR 0.0
	update_b 1 3
	updatepop_b 0 2
	jmp peek_byte_off_float_0
	:peek_byte_off_float_0
|	flds   (%eax,%ebx,1)
	instruction 217
	instruction 4
	instruction 24
|	fstp   %st(1)
	instruction 221
	instruction 217
 }

w`peek_elem_off_float :: !(Ptr Float) !Int !*World -> (!Float,!*World);
w`peek_elem_off_float p i w = code {
	pushR 0.0
	update_b 1 3
	updatepop_b 0 2
	jmp peek_elem_off_float_0
	:peek_elem_off_float_0
|	flds   (%eax,%ebx,4)
	instruction 217
	instruction 4
	instruction 152
|	fstp   %st(1)
	instruction 221
	instruction 217
 }

w`poke_float :: !(Ptr Float) !Float !*World -> (!PreludeTypes.(),!*World)
w`poke_float p v w = code {
	| fsts (%eax)
	instruction 217
	instruction 16
	pop_b 3
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_byte_off_float :: !(Ptr Float) !Int !Float !*World -> (!PreludeTypes.(),!*World)
w`poke_byte_off_float p i v w = code {
|	fsts  (%eax,%ebx,1)
	instruction	217
	instruction 20
	instruction 24
	pop_b 4
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_elem_off_float :: !(Ptr Float) !Int !Float !*World -> (!PreludeTypes.(),!*World)
w`poke_elem_off_float p i v w = code {
|	fsts  (%eax,%ebx,4)
	instruction	217
	instruction 20
	instruction 152
	pop_b 4
	buildh e_PreludeTypes_d`Unit 0
 }

w`peek_double :: !(Ptr Real) !*World -> (!Real,!*World);
w`peek_double p w = code {
	pushR 0.0
	update_b 1 2
	updatepop_b 0 1
	jmp peek_double_0
	:peek_double_0
	instruction 221
	instruction 0	| fldl (%eax)
	instruction 221
	instruction 217 | fstp   %st(1)
 }

w`poke_double :: !(Ptr Real) !Real !*World -> (!PreludeTypes.(),!*World)
w`poke_double p v w = code {
	| fstl (%eax)
	instruction 221
	instruction 16
	pop_b 4
	buildh e_PreludeTypes_d`Unit 0
 }

w`peek_byte_off_double :: !(Ptr Real) !Int !*World -> (!Real,!*World);
w`peek_byte_off_double p i w = code {
	pushR 0.0
	update_b 1 3
	updatepop_b 0 2
	jmp peek_byte_off_double_0
	:peek_byte_off_double_0
|	fldl   (%eax,%ebx,8)
	instruction 221
	instruction 4
	instruction 216
|	fstp   %st(1)
	instruction 221
	instruction 217
 }

w`peek_elem_off_double :: !(Ptr Real) !Int !*World -> (!Real,!*World);
w`peek_elem_off_double p i w = code {
	pushR 0.0
	update_b 1 3
	updatepop_b 0 2
	jmp peek_elem_off_double_0
	:peek_elem_off_double_0
|	fldl   (%eax,%ebx)
	instruction 221
	instruction 4
	instruction 24
|	fstp   %st(1)
	instruction 221
	instruction 217
 }

w`poke_byte_off_double :: !(Ptr Real) !Int !Real !*World -> (!PreludeTypes.(),!*World)
w`poke_byte_off_double p i v w = code {
|	fstl  (%eax,%ebx,1)
	instruction	221
	instruction 20
	instruction 24
	pop_b 4
	buildh e_PreludeTypes_d`Unit 0
 }

w`poke_elem_off_double :: !(Ptr Real) !Int !Real !*World -> (!PreludeTypes.(),!*World)
w`poke_elem_off_double p i v w = code {
|	fstl  (%eax,%ebx,8)
	instruction	221
	instruction 20
	instruction 224
	pop_b 4
	buildh e_PreludeTypes_d`Unit 0
 }
