implementation module List where

import Prelude
from StdMisc import abort

elemIndex :: (Eq a) => a -> ![a] -> Maybe Int
elemIndex y xs = elemIndex xs y `0
	where
		elemIndex :: ![a] -> a -> !Int -> Maybe Int | Eq a
		elemIndex (x:xs) y i
			| y==x
				= Just i
				= elemIndex xs y (i+`1)
		elemIndex [] y i
			= Nothing

elemIndices :: (Eq a) => a -> ![a] -> [Int]
elemIndices y xs = elemIndices xs y `0
	where
		elemIndices :: ![a] -> a -> !Int-> [Int] | Eq a
		elemIndices (x:xs) y i
			| y==x
				= i : elemIndices xs y (i+`1)
				= elemIndices xs y (i+`1)
		elemIndices [] y i
			= []

--find :: (a->Bool) -> [a] -> Maybe a
find p l :== find l
where
	find (e:l)
		| p e
			= Just e
			= find l
	find [] = Nothing

findIndex :: (a -> Bool) -> ![a] -> Maybe Int
findIndex p xs = findIndex xs `0 p
	where
		findIndex :: ![a] -> !Int -> (a->Bool) -> Maybe Int
		findIndex (x:xs) i p
			| p x
				= Just i
				= findIndex xs (i+`1) p
		findIndex [] i p
			= Nothing

findIndices :: (a -> Bool) -> ![a] -> [Int]
findIndices p xs = findIndices xs `0 p
	where
		findIndices :: ![a] -> !Int -> (a->Bool) -> [Int]
		findIndices (x:xs) i p
			| p x
				= i : findIndices xs (i+`1) p
				= findIndices xs (i+`1) p
		findIndices [] i p
			= []

nub :: (Eq a) => ![a] -> [a]
nub [] = []
nub (x:xs) = x : nub [e | e<-xs, e/=x]

nubBy :: (a -> a -> Bool) -> ![a] -> [a]
nubBy eq (x:xs) = x : nubBy eq [e | e<-xs, not (eq x e)]
nubBy eq [] = []

delete :: (Eq a) => a -> ![a] -> [a]
delete x l = Delete x l

deleteBy :: (a->a->Bool) -> a -> ![a] -> [a]
deleteBy eq x l = DeleteBy eq x l

DeleteBy eq x l :== delete_by x l
where
	delete_by x []
		= []
	delete_by x (y:ys)
		| eq x y
			= ys
			= y:delete_by x ys

Delete x l :== delete x l
where
	delete x []
		= []
	delete x (y:ys)
		| x==y
			= ys
			= y:delete x ys

infix 5 \\

(\\) :: (Eq a) => ![a] -> ![a] -> [a]
l1 \\ l2 = foldl (\ a b -> Delete b a) l1 l2

deleteFirstsBy :: (a -> a -> Bool) -> ![a] -> ![a] -> [a]
deleteFirstsBy eq l1 l2 = foldl (\l x -> DeleteBy eq x l) l1 l2

union :: (Eq a) => ![a] -> [a] -> [a]
union xs ys = xs ++ foldl (\l x -> Delete x l) (nub ys) xs

unionBy :: (a -> a -> Bool) -> ![a] -> [a] -> [a]
unionBy eq xs ys = xs ++ deleteFirstsBy eq (nubBy eq ys) xs

intersect :: (Eq a) => ![a] -> [a] -> [a]
intersect xs ys = [x | x<-xs, any_eq x ys]
	where
	any_eq x [] = False
	any_eq x (b : bs) = b==x || any_eq x bs

intersectBy :: (a -> a -> Bool) -> ![a] -> [a] -> [a]
intersectBy eq xs ys = [x | x <- xs, any (eq x) ys]

intersperse :: a -> ![a] -> [a]
intersperse sep [] = []
intersperse sep (l@[x]) = l
intersperse sep (x:xs) = x : sep : intersperse sep xs

-- transpose is lazy in both rows and columns,
-- and works for non-rectangular matrices
-- For example, transpose [[1,2],[3,4,5],[]] = [[1,3],[2,4],[5]]
-- Note that [h | (h:t) <- xss] is not the same as (map head xss)
-- because the former discards empty sublists inside xss
transpose :: ![[a]] -> [[a]]
transpose [] = []
transpose ([] : xss) = transpose xss
transpose ((x:xs) : xss) = (x : [h | (h:t) <- xss]) : transpose (xs : [t | (h:t) <- xss])

partition :: (a->Bool) -> ![a] -> ([a],[a])
partition p xs = partition xs
	where
		partition (a:x)
			# (ts,fs) = partition x
			| p a
				= (a:ts,fs)
				= (ts,a:fs)
		partition []
			= ([],[])

group :: (Eq a) => ![a] -> [[a]]
group [] = []
group (x:xs) = (x:ys) : group zs
					where (ys,zs) = span ((==) x) xs

groupBy :: (a -> a -> Bool) -> ![a] -> [[a]]
groupBy eq [] = []
groupBy eq (x:xs) = (x:ys) : groupBy eq zs
					where (ys,zs) = span (eq x) xs

inits :: ![a] -> [[a]]
inits [] = [[]]
inits (x:xs) = [] : [x:l | l<-inits xs]

tails :: ![a] -> [[a]]
tails [] = [[]]
tails xxs@(_:xs) = xxs : tails xs

isPrefixOf :: (Eq a) => ![a] -> [a] -> Bool
isPrefixOf [] _ = True
isPrefixOf _ [] = False
isPrefixOf (x:xs) (y:ys) = x==y && isPrefixOf xs ys

isSuffixOf :: (Eq a) => ![a] -> [a] -> Bool
isSuffixOf x y = reverse x `isPrefixOf` reverse y

mapAccumL :: (s -> a -> (s, b)) -> s -> ![a] -> (s, [b])
mapAccumL f s (x:xs)
	= let
		(s', y ) = f s x
		(s'',ys) = mapAccumL f s' xs
	  in
		(s'',y:ys)
wheremapAccumL f s []
	= (s, [])

mapAccumR :: (s -> a -> (s, b)) -> s -> ![a] -> (s, [b])
mapAccumR f s (x:xs)
	= let
		(s', ys) = mapAccumR f s xs
		(s'',y ) = f s' x
	  in
	  	(s'', y:ys)
mapAccumR f s []
	= (s, [])

unfoldr :: !(b -> Maybe (a,b)) -> b -> [a]
unfoldr f b
	= case f b of
		Nothing -> []
		Just (a,b) -> a : unfoldr f b


sort :: (Ord a) => ![a] -> [a]
sort l = mergeAll (pair l)
  where
	pair (x1:x2:xs)
		| x1>x2
			= [x2,x1]:pair xs
			= [x1,x2]:pair xs
	pair x = [x]

    mergeAll [x] = x
    mergeAll xs  = mergeAll (mergePairs xs)

    mergePairs (a:b:xs) = merge a b: mergePairs xs
    mergePairs xs       = xs

    merge as@(a:as') bs@(b:bs')
      | a > b	= b:merge as  bs'
      			= a:merge as' bs
    merge [] bs = bs
    merge as [] = as

sortBy :: (a -> a -> Ordering) -> ![a] -> [a]
sortBy cmp l = mergeAll (pair l)
  where
	pair (x1:x2:xs)
		| x1 `cmp` x2 == GT
			= [x2,x1]:pair xs
			= [x1,x2]:pair xs
	pair x = [x]

    mergeAll [x] = x
    mergeAll xs  = mergeAll (mergePairs xs)

    mergePairs (a:b:xs) = merge a b: mergePairs xs
    mergePairs xs       = xs

    merge as@(a:as') bs@(b:bs')
      | a `cmp` b == GT = b:merge as  bs'
      | otherwise       = a:merge as' bs
    merge [] bs         = bs
    merge as []         = as

insert :: (Ord a) => a -> ![a] -> [a]
insert x l = insertBy compare x l

insertBy :: (a -> a -> Ordering) -> a -> ![a] -> [a]
insertBy cmp x ys@(y:ys')
	= case cmp x y of
		GT -> y : insertBy cmp x ys'
		_  -> x : ys
insertBy cmp x [] = [x]

Insert x l :== insert x l
	where
		insert x ys@(y:ys')
			| x>y
				= y : insert x ys'
				= x : ys
		insert x []
			= [x]

insert`Int :: Int -> ![Int] -> [Int]
insert`Int x l = Insert x l

insert`Char :: Char -> ![Char] -> [Char]
insert`Char x l = Insert x l

insert`Double :: Double -> ![Double] -> [Double]
insert`Double x l = Insert x l

insert`Float :: Float -> ![Float] -> [Float]
insert`Float x l = Insert x l

maximumBy :: (a -> a -> Ordering) -> ![a] -> a
maximumBy cmp (x:xs) = foldl max x xs
	where
		max x y
			= case cmp x y of
				GT -> x
				_ -> y
maximumBy cmp []
	= abort `"List.maximumBy: empty list"

minimumBy :: (a -> a -> Ordering) -> ![a] -> a
minimumBy cmp (x:xs) = foldl min x xs
	where
		min x y
			= case cmp x y of
				GT -> y
				_ -> x
minimumBy cmp []
	= abort `"List.minimumBy: empty list"

zip4 :: ![a] -> [b] -> [c] -> [d] -> [(a,b,c,d)]
zip4 (a:as) (b:bs) (c:cs) (d:ds) = (a,b,c,d) : zip4 as bs cs ds
zip4 _ _ _ _ = []

zip5 :: ![a] -> [b] -> [c] -> [d] -> [e] -> [(a,b,c,d,e)]
zip5 (a:as) (b:bs) (c:cs) (d:ds) (e:es) = (a,b,c,d,e) : zip5 as bs cs ds es
zip5 _ _ _ _ _ = []

zip6 :: ![a] -> [b] -> [c] -> [d] -> [e] -> [f] -> [(a,b,c,d,e,f)]
zip6 (a:as) (b:bs) (c:cs) (d:ds) (e:es) (f:fs) = (a,b,c,d,e,f) : zip6 as bs cs ds es fs
zip6 _ _ _ _ _ _ = []

zip7 :: ![a] -> [b] -> [c] -> [d] -> [e] -> [f] -> [g] -> [(a,b,c,d,e,f,g)]
zip7 (a:as) (b:bs) (c:cs) (d:ds) (e:es) (f:fs) (g:gs) = (a,b,c,d,e,f,g) : zip7 as bs cs ds es fs gs
zip7 _ _ _ _ _ _ _ = []

zipWith4 :: (a->b->c->d->e) -> ![a]->[b]->[c]->[d]->[e]
zipWith4 z (a:as) (b:bs) (c:cs) (d:ds) = z a b c d : zipWith4 z as bs cs ds
zipWith4 _ _ _ _ _ = []

zipWith5 :: (a->b->c->d->e->f) -> ![a]->[b]->[c]->[d]->[e]->[f]
zipWith5 z (a:as) (b:bs) (c:cs) (d:ds) (e:es) = z a b c d e : zipWith5 z as bs cs ds es
zipWith5 _ _ _ _ _ _ = []

zipWith6 :: (a->b->c->d->e->f->g) -> ![a]->[b]->[c]->[d]->[e]->[f]->[g]
zipWith6 z (a:as) (b:bs) (c:cs) (d:ds) (e:es) (f:fs) = z a b c d e f : zipWith6 z as bs cs ds es fs
zipWith6 _ _ _ _ _ _ _ = []

zipWith7 :: (a->b->c->d->e->f->g->h) -> ![a]->[b]->[c]->[d]->[e]->[f]->[g]->[h]
zipWith7 z (a:as) (b:bs) (c:cs) (d:ds) (e:es) (f:fs) (g:gs) = z a b c d e f g : zipWith7 z as bs cs ds es fs gs
zipWith7 _ _ _ _ _ _ _ _ = []

unzip4 :: ([(a,b,c,d)] -> ([a],[b],[c],[d]))
unzip4 = unzip4 where
	unzip4 ((a,b,c,d):l) = let (as,bs,cs,ds) = unzip4 l in (a:as,b:bs,c:cs,d:ds)
	unzip4 [] = ([],[],[],[])

unzip5 :: ([(a,b,c,d,e)] -> ([a],[b],[c],[d],[e]))
unzip5 = unzip5 where
	unzip5 ((a,b,c,d,e):l) = let (as,bs,cs,ds,es) = unzip5 l in (a:as,b:bs,c:cs,d:ds,e:es)
	unzip5 [] = ([],[],[],[],[])

unzip6 :: ([(a,b,c,d,e,f)] -> ([a],[b],[c],[d],[e],[f]))
unzip6 = unzip6 where
	unzip6 ((a,b,c,d,e,f):l) = let (as,bs,cs,ds,es,fs) = unzip6 l in (a:as,b:bs,c:cs,d:ds,e:es,f:fs)
	unzip6 [] = ([],[],[],[],[],[])

unzip7 :: ([(a,b,c,d,e,f,g)] -> ([a],[b],[c],[d],[e],[f],[g]))
unzip7 = unzip7 where
	unzip7 ((a,b,c,d,e,f,g):l) = let (as,bs,cs,ds,es,fs,gs) = unzip7 l in (a:as,b:bs,c:cs,d:ds,e:es,f:fs,g:gs)
	unzip7 [] = ([],[],[],[],[],[],[])

genericLength :: (Integral a) => ![b] -> a
genericLength [] = 0
genericLength (x:xs) = 1 + genericLength xs

genericTake :: (Integral a) => a -> ![b] -> [b]
genericTake _ [] = []
genericTake 0 _ = []
genericTake n (x:xs)
	| n > 0 = x : genericTake (n-1) xs
			= genericTake_negative_argument

genericTake_negative_argument
	= abort `"List.genericTake: negative argument"

genericDrop :: (Integral a) => !a -> ![b] -> [b]
genericDrop 0 xs = xs
genericDrop _ [] = []
genericDrop n (_:xs)
	| n > 0 = genericDrop (n-1) xs
			= genericDrop__negative_argument

genericDrop__negative_argument
	= abort `"List.genericDrop: negative argument"

genericSplitAt :: (Integral a) => !a -> [b] -> ([b],[b])
genericSplitAt 0 xs = ([],xs)
genericSplitAt _ [] = ([],[])
genericSplitAt n (x:xs)
	| n > 0 = (x:xs',xs'')
			= genericSplitAt_negative_argument
	where
		(xs',xs'') = genericSplitAt (n-1) xs

genericSplitAt_negative_argument
	= abort `"List.genericSplitAt: negative argument"

genericIndex :: (Integral a) => ![b] -> !a -> b
genericIndex (x:_) 0 = x
genericIndex (_:xs) n
	| n > 0 = genericIndex xs (n-1)
			= genericIndex_negative_argument
genericIndex _ _
	= genericIndex_too_large

genericIndex_negative_argument
	= abort `"List.genericIndex: negative argument"

genericIndex_too_large
	= abort `"List.genericIndex: index too large"

genericReplicate :: (Integral a) => a -> b -> [b]
genericReplicate n x = genericTake n (repeat x)
