implementation module Monad where

class (Monad m) => MonadPlus m where
	mzero :: m a
	mplus :: m a -> m a -> m a

instance MonadPlus Maybe where
	mzero = Nothing

	mplus :: !(Maybe a) -> Maybe a -> Maybe a
	Nothing `mplus` ys = ys
	xs `mplus` ys = xs

instance MonadPlus [] where
	mzero = []

	mplus :: ![a] -> [a] -> [a]
	xs `mplus` ys = xs ++ ys

msum :: MonadPlus m => ![m a] -> m a
msum xs = foldr mplus mzero xs

join :: (Monad m) => m (m a) -> m a
join x = x >>= id

--when :: (Monad m) => Bool -> m () -> m ()
when p s :== if p then s else return ()

--unless :: (Monad m) => Bool -> m () -> m ()
unless p s :== if p then return () else s

ap :: (Monad m) => m (a -> b) -> m a -> m b
ap m1 m2 = do { x1 <- m1; x2 <- m2; return (x1 x2) }

//guard :: MonadPlus m => Bool -> m ()
guard p :== if p then return () else mzero

mapAndUnzipM :: (Monad m) => (a -> m (b,c)) -> [a] -> m ([b], [c])
mapAndUnzipM f xs = sequence (map f xs) >>= return . unzip

zipWithM :: (Monad m) => (a -> b -> m c) -> [a] -> [b] -> m [c]
zipWithM f xs ys = sequence (zipWith f xs ys)

zipWithM_ :: (Monad m) => (a -> b -> m c) -> [a] -> [b] -> m ()
zipWithM_ f xs ys = sequence_ (zipWith f xs ys)

foldM :: (Monad m) => (a -> b -> m a) -> a -> ![b] -> m a
foldM f a [] = return a
foldM f a (x:xs) = f a x >>= \ y -> foldM f y xs

filterM :: Monad m => (a -> m Bool) -> ![a] -> m [a]
filterM p []
	= return []
filterM p (x:xs)
	= do {	b <- p x;
			ys <- filterM p xs;
			return (if b then (x:ys) else ys)
	  }

liftM2  :: (Monad m) => (a1 -> a2 -> r) -> m a1 -> m a2 -> m r
liftM2 f m1 m2
	= do { x1 <- m1; x2 <- m2; return (f x1 x2) }

liftM3  :: (Monad m) => (a1 -> a2 -> a3 -> r) -> m a1 -> m a2 -> m a3 -> m r
liftM3 f m1 m2 m3
	= do { x1 <- m1; x2 <- m2; x3 <- m3; return (f x1 x2 x3) }

liftM4  :: (Monad m) => (a1 -> a2 -> a3 -> a4 -> r) -> m a1 -> m a2 -> m a3 -> m a4 -> m r
liftM4 f m1 m2 m3 m4
	= do { x1 <- m1; x2 <- m2; x3 <- m3; x4 <- m4; return (f x1 x2 x3 x4) }

liftM5  :: (Monad m) => (a1 -> a2 -> a3 -> a4 -> a5 -> r) -> m a1 -> m a2 -> m a3 -> m a4 -> m a5 -> m r
liftM5 f m1 m2 m3 m4 m5
	= do { x1 <- m1; x2 <- m2; x3 <- m3; x4 <- m4; x5 <- m5; return (f x1 x2 x3 x4 x5) }
