definition module Prelude where

import PreludeTypes
import PreludeInline
import PreludeInlineFunctions
import PreludeInlineFromIntegral
import PreludeInlineRealToFrac
import _PreludeEnum
import PreludeInteger
import PreludeFloat
import PreludeTuples
import PreludeList
import PreludeShow
import PreludeRead
import PreludeIO
import PreludeRatio

from StdBool import not,&&,||
from StdTuple import fst,snd,curry,uncurry
from StdFunc import id,const,flip
from StdEnum import not
from StdMisc import qualified abort

from StdChar import isPrint

seq infixr 0 :: !.a -> !.b -> .b

(.) infixr  9 // ::  u:(.a -> .b) u:(.c -> .a) -> u:(.c -> .b) // Function composition
(.) f g :== \ x -> f (g x)

($) infixr 0
($) f g :== f g

($!) infixr 0
($!) f g :== seq g (f g)

-- until :: !(a -> Bool) -> (a -> a) -> a -> a
until p f x :== until x
	where
		until x
			| p x
				= x
				= until (f x)

asTypeOf :: !.a -> .a -> .a

(^) infixr 8 :: (Num a, Integral b) => a -> !b -> a
		special a=Int,b=Int:==power`Int_Int
				a=Int,b=Integer:==power`Int_Integer
				a=Integer,b=Int:==power`Integer_Int
				a=Integer,b=Integer:==power`Integer_Integer
				a=Double,b=Int:==power`Double_Int
				a=Double,b=Integer:==power`Double_Integer
				a=Float,b=Int:==power`Float_Int
				a=Float,b=Integer:==power`Float_Integer

power`Int_Int :: !Int -> !Int -> Int
power`Int_Integer :: !Int -> !Integer -> Int
power`Integer_Int :: !Integer -> !Int -> Integer
power`Integer_Integer :: !Integer -> !Integer -> Integer
power`Double_Int :: !Double -> !Int -> Double
power`Double_Integer :: !Double -> !Integer -> Double
power`Float_Int :: !Float -> !Int -> Float
power`Float_Integer :: !Float -> !Integer -> Float

(^^) infixr 8 :: (Fractional a,Integral b) => a -> !b -> a
		special a=Double,b=Int
				a=Double,b=Integer
				a=Float,b=Int
				a=Float,b=Integer

gcd :: (Integral a) => !a -> a -> a
		special a=Int:==gcd`Int
				a=Integer:==gcd`Integer

gcd`Int :: !Int -> !Int -> Int
gcd`Integer :: !Integer -> !Integer -> Integer

lcm :: (Integral a) => a -> !a -> a
	special a=Int; a=Integer

--maybe :: b -> (a -> b) -> Maybe a -> b
maybe n f m :== case m of
				Nothing -> n
				(Just x) -> f x

instance (Eq a) =>
  Eq (Maybe a) where
	(==) :: !(Maybe a) -> !(Maybe a) -> Bool | Eq a
	(/=) :: !(Maybe a) -> !(Maybe a) -> Bool | Eq a

instance (Ord a) =>
  Ord (Maybe a) where
	(<) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a
	(>) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a
	(>=) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a
	(<=) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a

instance Functor Maybe where
	fmap :: (a -> b) -> !(Maybe a) -> Maybe b

//either :: (a -> c) -> (b -> c) -> Either a b -> c
either f g e
	:== case e of
		(Left x) -> f x
		(Right y) -> g y

instance (Eq a,Eq b) =>
  Eq (Either a b) where
	(==) :: (Eq a,Eq b) => !(Either a b) -> !(Either a b) -> Bool
	(/=) :: (Eq a,Eq b) => !(Either a b) -> !(Either a b) -> Bool

instance (Ord a,Ord b) =>
  Ord (Either a b) where
	(<) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool
	(>) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool
	(<=) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool
	(>=) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool

instance Functor IO

class
  Monad m where
	return :: a -> m a
    (>>=) infixl 1 :: m a -> (a -> m b) -> m b
	(>>) infixl 1 :: m a -> m b -> m b
	fail :: [Char] -> m a
	(>>) m k = m >>= \_ -> k
	fail s = error s

instance Monad IO where
	return :: a -> IO a
	return v
		= IO return_io
		where
			return_io w
				= (v,w)
    (>>=) :: IO a -> (a -> IO b) -> IO b
	(>>=) (IO f) g
		= IO bind_io
		where
			bind_io w
				# (a,w) = f w
				#! w = w
				# (IO h) = g a
			  	= h w

	(>>)   :: IO a -> IO b -> IO b
	(>>) (IO f) (IO h)
		= IO then_io
		where
			then_io w
				# (a,w) = f w
				#! w=w
				= h w

instance Enum Int where
	toEnum :: !Int -> Int
	fromEnum :: !Int -> Int

instance Bounded Int

instance Bounded Bool

instance Bounded Char

instance (Eq a) =>
  Eq [a] where
	(==) :: (Eq a) => ![a] -> ![a] -> Bool
	(/=) :: (Eq a) => ![a] -> ![a] -> Bool

instance (Ord a) =>
  Ord [a] where
	(<) :: (Ord a) => ![a] -> ![a] -> Bool
	(>) :: (Ord a) => ![a] -> ![a] -> Bool
	(<=) :: (Ord a) => ![a] -> ![a] -> Bool
	(>=) :: (Ord a) => ![a] -> ![a] -> Bool

instance Functor [] where
	fmap :: (a -> b) -> ![a] -> [b]

instance Monad []

error s :== StdMisc.abort (char_list`to_string s)

char_list`to_string :: ![Char] -> .{#Char}

undefined :: .a

instance Eq () where
	(==) :: !() -> !() -> Bool
	(/=) :: !() -> !() -> Bool

instance Ord () where
	compare :: !() -> !() -> Ordering
	(<=) :: !() -> !() -> Bool
	(<) :: !() -> !() -> Bool
	(>=) :: !() -> !() -> Bool
	(>) :: !() -> !() -> Bool
	min :: !() -> !() -> ()
	max :: !() -> !() -> ()

instance Bounded ()

instance Enum () where
	toEnum :: !Int -> ()
	fromEnum :: !() -> Int

instance Bounded Ordering

instance Ord Ordering where
	compare :: !Ordering -> !Ordering -> Ordering
	(<=) :: !Ordering -> !Ordering -> Bool
	(<) :: !Ordering -> !Ordering -> Bool
	(>=) :: !Ordering -> !Ordering -> Bool
	(>) :: !Ordering -> !Ordering -> Bool
	min :: !Ordering -> !Ordering -> Ordering
	max :: !Ordering -> !Ordering -> Ordering

instance Ord Bool where
	compare :: !Bool -> !Bool -> Ordering
	(<=) :: !Bool -> !Bool -> Bool
	(<) :: !Bool -> !Bool -> Bool
	(>=) :: !Bool -> !Bool -> Bool
	(>) :: !Bool -> !Bool -> Bool
	min :: !Bool -> !Bool -> Bool
	max :: !Bool -> !Bool -> Bool

instance Enum Bool where
	toEnum :: !Int -> Bool
	fromEnum :: !Bool -> Int

instance Enum Ordering where
	toEnum :: !Int -> Ordering
	fromEnum :: !Ordering -> Int

(COMBINE_COMPARE) infixr 8
(COMBINE_COMPARE) c1 c2
	:==case c1 of
		EQ -> c2
		LT -> LT
		GT -> GT

sequence :: (Monad m) => ([m a] -> m [a])
sequence_ :: (Monad m) => ([m a] -> m ()) special m=IO
mapM :: (Monad m) => (a -> m b) -> ![a] -> m [b]
mapM_ :: (Monad m) => (a -> m b) -> ![a] -> m () special m=IO
(=<<) infixr 1 :: Monad m => (a -> m b) -> m a -> m b

enumFrom`Double :: Double -> [Double]
enumFrom`Float :: Float -> [Float]
enumFromTo`Double :: !Double -> !Double -> [Double]
enumFromTo`Float :: !Float -> !Float -> [Float]
enumFromThen`Double :: Double -> Double -> [Double]
enumFromThen`Float :: Float -> Float -> [Float]
enumFromThenTo`Double :: !Double -> !Double -> !Double -> [Double]
enumFromThenTo`Float :: !Float -> !Float -> !Float -> [Float]
