implementation module Prelude where

from StdOverloaded import
	class zero (..),class one (..)
from StdOverloaded import qualified
	class + (..),class - (..),class * (..)
from StdBool import not,&&,||
from StdInt import
	instance + Int,instance - Int,instance one Int,instance * Int,IF_INT_64_OR_32
from StdChar import isSpace,isDigit,isUpper,isAlpha,isAlphanum,isOctDigit,isHexDigit,digitToInt
from StdString import class toString (..),instance toString Int,instance toString Double
import StdMisc
from StdList import foldr,++,map,take
from StdEnum import inc
import StdArray

import PreludeInline // PreludeTypes
import PreludeInteger
import PreludeList
import PreludeShow
import PreludeIO (ioError,userError)

seq infixr 0 :: !.a -> !.b -> .b
seq a b = abort `"Prelude.seq"

(^) infixr 8 :: (Num a, Integral b) => a -> !b -> a
x ^ 0 = 1
x ^ n | n > 0 = f x (n-1) x
  where
  	f :: (Num a, Integral b) => !a -> !b -> !a -> a
	f _ 0 y = y
	f x n y = g x n y where
	 	g :: (Num a, Integral b) => !a -> !b -> !a -> a
		g x n y	| n `rem` 2==0	= g (x*x) (n `quot` 2) y // using mod and div would be faster
				| otherwise		= f x (n-1) (x*y)
_ ^ _ = negative_exponent_error

power`Int_Int :: !Int -> !Int -> Int
power`Int_Int x n = power`Num_Int x n

power`Int_Integer :: !Int -> !Integer -> Int
power`Int_Integer x n = power`Num_Integer x n power`Int_Int

power`Integer_Int :: !Integer -> !Int -> Integer
power`Integer_Int x n = power`Num_Int x n

power`Integer_Integer :: !Integer -> !Integer -> Integer
power`Integer_Integer x n = power`Num_Integer x n power`Integer_Int

power`Double_Int :: !Double -> !Int -> Double
power`Double_Int x n = power`Num_Int x n

power`Double_Integer :: !Double -> !Integer -> Double
power`Double_Integer x n = power`Num_Integer x n power`Double_Int

power`Float_Int :: !Float -> !Int -> Float
power`Float_Int x n = power`Num_Int x n

power`Float_Integer :: !Float -> !Integer -> Float
power`Float_Integer x n = power`Num_Integer x n power`Float_Int

power`Num_Int x n
	:== if (n>0)
			(power_f x (n-1) x)
		(if (n==0)
	  		1
			negative_exponent_error)

power`Num_Integer x n power_num_int
	:==	if (size n~integer_a==0)
			(power_num_int x n~integer_s)
	 	(if (n~integer_s>=0)
			(power_f x (n-1) x)
			negative_exponent_error)

power_f :: (Num a, Integral b) => !a -> !b -> !a -> a
power_f _ 0 y = y
power_f x n y = g x n y
  where
 	g :: (Num a, Integral b) => !a -> !b -> !a -> a
	g x n y
		| even n
			= g (x*x) (n `div` 2) y
			= power_f x (n-1) (x*y)

negative_exponent_error = abort `"Prelude.^: negative exponent"

(^^) infixr 8 :: (Fractional a, Integral b) => a -> !b -> a
x ^^ n = if n >= 0 then x^n else recip (x^(-n))

gcd :: (Integral a) => !a -> a -> a
gcd 0 0 = gcd_0_0_error
gcd x y = gcd` (abs x) (abs y)
	where
		gcd` x 0 = x
		gcd` x y = gcd` y (x `rem` y)

gcd`Int :: !Int -> !Int -> Int
gcd`Int 0 0 = gcd_0_0_error
gcd`Int x y = gcd` (abs x) (abs y)
	where
		gcd` x 0 = x
		gcd` x y = gcd` y (x `rem` y)

MaxNegInt :== IF_INT_64_OR_32 `0x8000000000000000 `0x80000000

gcd`Integer :: !Integer -> !Integer -> Integer
gcd`Integer x y
	| size x~integer_a==0 && size y~integer_a==0 && x~integer_s/=MaxNegInt && y~integer_s/=MaxNegInt
		= {integer_a={},integer_s=gcd`Int x~integer_s y~integer_s}
		= gcd` (abs x) (abs y)
	where
		gcd` x 0 = x
		gcd` x y = gcd` y (x `rem` y)

gcd_0_0_error = abort `"Prelude.gcd: gcd 0 0 is undefined"

lcm :: (Integral a) => a -> !a -> a
lcm _ 0 = 0
lcm 0 _ = 0
lcm x y = abs ((x `quot` (gcd x y)) * y)

class Monad m where
	return :: a -> m a
    (>>=) infixl 1 :: m a -> (a -> m b) -> m b
	(>>) infixl 1 :: m a -> m b -> m b
	fail :: [Char] -> m a
	(>>) m k = m >>= \_ -> k
	fail s = error s

instance Monad IO where
	return v
		= IO return_io
		where
			return_io w
				= (v,w)
	(>>=) (IO f) g
		= IO bind_io
		where
			bind_io w
				# (a,w) = f w
				#! w = w
				# (IO h) = g a
			  	= h w
	(>>) (IO f) (IO h)
		= IO then_io
		where
			then_io w
				# (a,w) = f w
				#! w = w
				= h w
	fail s
		= ioError (userError s)

sequence :: (Monad m) => [m a] -> m [a]
sequence => foldr mcons (return [])
	where mcons p q = p >>= \x -> q >>= \y -> return (x:y)

sequence_ :: (Monad m) => [m a] -> m ()
sequence_ => foldr (>>) (return ())

mapM :: (Monad m) => (a -> m b) -> ![a] -> m [b]
mapM f as = sequence (map f as)

mapM_ :: (Monad m) => (a -> m b) -> ![a] -> m ()
mapM_ f as = sequence_ (map f as)

infixr 1 =<<
(=<<) :: Monad m => (a -> m b) -> m a -> m b
f =<< x = x >>= f

instance (Eq a) =>
  Eq (Maybe a) where
	(==) :: !(Maybe a) -> !(Maybe a) -> Bool | Eq a
	Just a == Just b = a==b
	Nothing == Nothing = True
	_ == _ = False

	(/=) :: !(Maybe a) -> !(Maybe a) -> Bool | Eq a
	Just a /= Just b = a/=b
	Nothing /= Nothing = False
	_ /= _ = True

instance (Ord a) =>
  Ord (Maybe a) where
	(<) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a
	Just a < Just b = a<b
	Nothing < Just _ = True
	_ < _ = False

	(>) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a
	Just a > Just b = a>b
	Just _ > Nothing = True
	_ > _ = False

	(>=) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a
	Just a >= Just b = a>b
	Nothing >= Just _ = False
	_ >= _ = True

	(<=) :: !(Maybe a) -> !(Maybe a) -> Bool | Ord a
	Just a <= Just b = a<=b
	Just _ <= Nothing = False
	_ <= _ = True

instance Functor Maybe where
	fmap :: (a -> b) -> !(Maybe a) -> Maybe b
	fmap f Nothing = Nothing
	fmap f (Just x) = Just (f x)

instance (Eq a,Eq b) =>
  Eq (Either a b)
	where
	(==) :: (Eq a,Eq b) => !(Either a b) -> !(Either a b) -> Bool
	Left a == Left b = a == b
	Right a == Right b = a == b
	_ == _ = False

	(/=) :: (Eq a,Eq b) => !(Either a b) -> !(Either a b) -> Bool
	Left a /= Left b = a /= b
	Right a /= Right b = a /= b
	_ /= _ = True

instance (Ord a,Ord b) =>
  Ord (Either a b)
	where
	(<) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool
	Left a < Left b = a < b
	Left a < _ = True
	Right a < Right b = a < b
	Right a < _ = False

	(>) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool
	Left a > Left b = a > b
	Left a > _ = False
	Right a > Right b = a > b
	Right a > _ = True

	(<=) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool
	Left a <= Left b = a <= b
	Left a <= _ = True
	Right a <= Right b = a <= b
	Right a <= _ = False

	(>=) :: (Ord a,Ord b) => !(Either a b) -> !(Either a b) -> Bool
	Left a >= Left b = a >= b
	Left a >= _ = False
	Right a >= Right b = a >= b
	Right a >= _ = True

instance Functor IO where
	fmap f x = x >>= (return . f)

instance (Eq a) =>
  Eq [a] where
	(==) :: (Eq a) => ![a] -> ![a] -> Bool
	[] == []
		= True
	[] == _
		= False
	_:_ == []
		= False
	a:as == b:bs 
		= a == b && as == bs

	(/=) :: (Eq a) => ![a] -> ![a] -> Bool
	[] /= []
		= False
	[] /= _
		= True
	_:_ /= []
		= True
	a:as /= b:bs 
		= a /= b || as /= bs

instance (Ord a) =>
  Ord [a]
	where
	(<) :: (Ord a) => ![a] -> ![a] -> Bool
	[] < []
		= False
	[] < _
		= True
	(_:_) < []
		= False
	(a:as) < (b:bs)
		= a < b || (a==b && as < bs)

	(>) :: (Ord a) => ![a] -> ![a] -> Bool
	[] > []
		= False
	[] > _
		= False
	(_:_) > []
		= True
	(a:as) > (b:bs)
		= a > b || (a==b && as > bs)

	(<=) :: (Ord a) => ![a] -> ![a] -> Bool
	[] <= []
		= True
	[] <= _
		= True
	(_:_) <= []
		= False
	(a:as) <= (b:bs)
		= a < b || (a==b && as <= bs)

	(>=) :: (Ord a) => ![a] -> ![a] -> Bool
	[] >= []
		= True
	[] >= _
		= False
	(_:_) >= []
		= True
	(a:as) >= (b:bs)
		= a > b || (a==b && as >= bs)

undefined :: .a
undefined => abort `"Prelude.undefined"

isAlphaNum c :== isAlphanum c

-- until p f yields the result of applying f until p holds.
-- until :: !(a -> Bool) -> (a -> a) -> a -> a
until p f x :== until x
	where
		until x
			| p x
				= x
				= until (f x)

-- asTypeOf is a type-restricted version of const. It is usually used
-- as an infix operator, and its typing forces its first argument
-- (which is usually overloaded) to have the same type as the second.
asTypeOf :: !.a -> .a -> .a
asTypeOf x y = x

instance Enum Int where
	toEnum :: !Int -> Int
	toEnum i = i

	fromEnum :: !Int -> Int
	fromEnum i = i

instance Bounded Int where
	minBound = 0x80000000
	maxBound = 0x7fffffff

instance Enum Bool where
	toEnum :: !Int -> Bool
	toEnum 0 = False
	toEnum 1 = True

	fromEnum :: !Bool -> Int
	fromEnum False = 0
	fromEnum True = 1

instance Bounded Bool where
	minBound = False
	maxBound = True

instance Bounded Char where
	minBound = '\x0';
	maxBound = '\xff';

instance Functor [] where
	fmap :: (a -> b) -> ![a] -> [b]
	fmap f l = map f l

instance Monad [] where
	m >>= k = concat (map k m)
	return x = [x]
	fail s = []

char_list`to_string :: ![Char] -> .{#Char}
char_list`to_string l = {c;;c<-l}

instance Eq () where
	(==) :: !() -> !() -> Bool
	_ == _ = True
	(/=) :: !() -> !() -> Bool
	_ /= _ = False

instance Ord () where
	compare :: !() -> !() -> Ordering
	compare a b = EQ

	(<=) :: !() -> !() -> Bool
	x <= y = True

	(<) :: !() -> !() -> Bool
	x < y  = False

	(>=) :: !() -> !() -> Bool
	x >= y = True

	(>) :: !() -> !() -> Bool
	x > y  = False

	min :: !() -> !() -> ()
	min x y  = ()

	max :: !() -> !() -> ()
	max x y = ()

instance Bounded () where
	minBound = ()
	maxBound = ()

instance Enum () where
	toEnum :: !Int -> ()
	toEnum 0 = ()

	fromEnum :: !() -> Int
	fromEnum () = 0

instance Ord Bool where
	compare :: !Bool -> !Bool -> Ordering
	compare False False = EQ
	compare False True = LT
	compare True False = GT
	compare True True = EQ

	(<=) :: !Bool -> !Bool -> Bool
	x <= y = not x || y

	(<) :: !Bool -> !Bool -> Bool
	x < y  = not x && y

	(>=) :: !Bool -> !Bool -> Bool
	x >= y = x || not y

	(>) :: !Bool -> !Bool -> Bool
	x > y = x && not y

	min :: !Bool -> !Bool -> Bool
	min x y = x && y

	max :: !Bool -> !Bool -> Bool
	max x y = x || y

instance Bounded Ordering where
	minBound = LT;
	maxBound = GT

instance Ord Ordering where
	compare :: !Ordering -> !Ordering -> Ordering
	compare (Ordering a) (Ordering b) = compare a b

	(<=) :: !Ordering -> !Ordering -> Bool
	Ordering x <= Ordering y = x<=y

	(<) :: !Ordering -> !Ordering -> Bool
	Ordering x < Ordering y  = x<y

	(>=) :: !Ordering -> !Ordering -> Bool
	Ordering x >= Ordering y = x>=y

	(>) :: !Ordering -> !Ordering -> Bool
	Ordering x > Ordering y = x>y

	min :: !Ordering -> !Ordering -> Ordering
	min (Ordering x) (Ordering y) = Ordering (min x y)

	max :: !Ordering -> !Ordering -> Ordering
	max (Ordering x) (Ordering y) = Ordering (max x y)

instance Enum Ordering where
	toEnum :: !Int -> Ordering
	toEnum i
		| i>= 0 && i<=2
			= Ordering (i-1)

	fromEnum :: !Ordering -> Int
	fromEnum (Ordering i) = i+1

enumFrom`Double :: Double -> [Double]
enumFrom`Double x = x : enumFrom`Double (x+1)

enumFrom`Float :: Float -> [Float]
enumFrom`Float x = x : enumFrom`Float (x+1)

enumFromTo`Double :: !Double -> !Double -> [Double]
enumFromTo`Double n m = enumFromTo`Double n (m+0.5)
	where
		enumFromTo`Double x u
			| x<=u
				= x : enumFromTo`Double (x+1) u
				= []

enumFromTo`Float :: !Float -> !Float -> [Float]
enumFromTo`Float n m = enumFromTo`Float n (m+0.5)
	where
		enumFromTo`Float x u
			| x<=u
				= x : enumFromTo`Float (x+1) u
				= []

numericEnumFromThenToUp n d u
	| n<=u
		#! n`=n+d
		= n : numericEnumFromThenToUp n` d u
		= []

numericEnumFromThenToDown n d l
	| n>=l
		#! n`=n+d
		= n : numericEnumFromThenToDown n` d l
		= []

enumFromThen`Double :: Double -> Double -> [Double]
enumFromThen`Double n m = numericEnumFromThen n (m-n)

enumFromThen`Float :: Float -> Float -> [Float]
enumFromThen`Float n m = numericEnumFromThen n (m-n)

numericEnumFromThen x d
	= x : numericEnumFromThen (x+d) d

enumFromThenTo`Double :: !Double -> !Double -> !Double -> [Double]
enumFromThenTo`Double n n' m
	# d = n'-n
	| n'>=n
		= numericEnumFromThenToUp n d (m+0.5*d)
		= numericEnumFromThenToDown n d (m+0.5*d)

enumFromThenTo`Float :: !Float -> !Float -> !Float -> [Float]
enumFromThenTo`Float n n' m
	# d = n'-n
	| n'>=n
		= numericEnumFromThenToUp n d (m+0.5*d)
		= numericEnumFromThenToDown n d (m+0.5*d)
