implementation module PreludeIO where

import PreludeInternalIOError
import PreludeInternal (string_to_char_list_append,char_list_to_string)
import StdFile,StdArray

from StdInt import qualified instance + Int,instance one Int
from StdMisc import qualified abort
from StdString import class +++(+++),instance +++ {#Char}

instance Show IOError where
	showsPrec _ io_error = string_to_char_list_append (ioeGetError`String io_error)

instance Eq IOError where
	(==) :: !IOError -> !IOError -> Bool
	DoesNotExistError == DoesNotExistError = True
	AlreadyExistsError == AlreadyExistsError = True
	PermissionError == PermissionError = True
	EOFError == EOFError = True
	FullError == FullError = True
	UserError s1 == UserError s2 = s1==s2
	_ == _ = False

	(/=) :: !IOError -> !IOError -> Bool
	DoesNotExistError /= DoesNotExistError = False
	AlreadyExistsError /= AlreadyExistsError = False
	PermissionError /= PermissionError = False
	EOFError /= EOFError = False
	FullError /= FullError = False
	UserError s1 /= UserError s2 = s1/=s2
	_ /= _ = True

ioError :: IOError -> IO a
ioError e = IO (io_error_world e)

io_error_world :: IOError -> !*World -> (a,!*World)
io_error_world e w
	# exh@(exh0,exh1,exh2) = get_exception_handler
	| exh0==0
		= StdMisc.abort (`"ioError: "+++ioeGetError`String e)
		#! (r,_,_) = jmp_exception_handler exh e w
		= seq r (StdMisc.abort `"ioError")

userError :: String -> IOError
userError str = UserError str

catch :: IO a -> (IOError -> IO a) -> IO a
catch (IO f) h
	= IO (catch_world f h)

catch_world :: !.(*World -> (.a,*World)) -> (IOError -> IO .a) -> !*World -> (.a,!*World)
catch_world f h w
	= catch_world_ f w h

catch_world_ :: !.(*World -> (.a,*World)) -> !*World -> (IOError -> IO .a) -> (.a,!*World)
catch_world_ f w h
	# old_exception_handler@(old_exception_handler0,_,_) = get_exception_handler
	| old_exception_handler0/=old_exception_handler0
		= abort_catch
	# (r,error,exception_w) = set_exception_handler
	| r==0
		# (a,w) = f w
		#! w=w
		| restore_exception_handler old_exception_handler==0
			= (a,w)
			= const (const abort_catch exception_w) h // use exception_w and h to prevent optimization
		| restore_exception_handler old_exception_handler==0
			# (IO hf) = h error
			= hf exception_w
			= abort_catch
  where
	abort_catch :: (.a,!*World)
	abort_catch
		= StdMisc.abort `"catch"

set_exception_handler :: (!Int,IOError,!*World)
set_exception_handler = `code {
	pushI 0
	pushI 0
	:set_exception_handler_0
	instruction 0x89
	instruction 0xf0	| mov    %esi,%eax
	instruction 0x89
	instruction 0xe3	| mov    %esp,%ebx
	:set_exception_handler_1
	push_b 2
	fillcaf ePreludeIO_exception_handler 0 3
	push_b_a 0
	pop_b 3
	.o 1 0
	:set_exception_handler_3
	pushI 0
	pop_a 1
	buildh _Nil 0
	push_a 0
	.caf ePreludeIO_exception_handler 0 3
}

get_exception_handler :: (!Int,!Int,!Int)
get_exception_handler = `code {
	pushcaf ePreludeIO_exception_handler 0 3
}

jmp_exception_handler :: !(!Int,!Int,!Int) -> IOError -> !*World -> (!Int,IOError,!*World)
jmp_exception_handler x e w = `code {
	instruction 0x8b
	instruction 0x34
	instruction 0x24	| mov    (%esp),%esi
	instruction 0x89
	instruction 0xc4	| mov    %eax,%esp
	:jmp_exception_handler_0
	update_b 0 2
	pop_b 2
	pushI 1
}

restore_exception_handler :: !(!Int,!Int,!Int) -> Int
restore_exception_handler x = `code {
	fillcaf ePreludeIO_exception_handler 0 3
	push_b_a 0
	pop_b 3
	:restore_exception_handler_0
	pushI 0
	pop_a 1
}

putChar :: Char -> IO ()
putChar c
	= IO (put_char_world c)

put_char_world :: !Char -> !*World -> (!(),!*World)
put_char_world c w = `code {
	ccall stdout_write_char "I:V:A"
	buildh e_PreludeTypes_d`Unit 0
}

putStr :: String -> IO ()
putStr s
	= IO (put_string_world s)
  where
	put_string_world :: ![Char] -> !*World -> (!(),!*World)
	put_string_world (c:cs) w
		# (_,w) = put_char_world c w 
		= put_string_world cs w
	put_string_world [] w
		= ((),w);

putStrLn :: String -> IO ()
putStrLn s
	= IO (put_string_line_world s)
  where
	put_string_line_world :: ![Char] -> !*World -> (!(),!*World)
	put_string_line_world (c:cs) w
		# (_,w) = put_char_world c w 
		= put_string_line_world cs w
	put_string_line_world [] w
		= put_char_world '\n' w

//print :: Show a => a -> IO () 
print x :== putStrLn (show x)

getChar :: IO Char
getChar = IO get_char_world
  where
	get_char_world :: !*World -> (!Char,!*World)
	get_char_world w
		# (c,w) = stdin_read_char_world w
		| c>=0
			= (toEnum c,w)
			= io_error_world EOFError w 

stdin_read_char_world :: !*World -> (!Int,!*World)
stdin_read_char_world w = `code {
	ccall stdin_read_char ":I:A"
}

stdin_read_char :: Int
stdin_read_char = `code {
	ccall stdin_read_char ":I"
}

getLine :: IO [Char]
getLine
	= IO get_line`World

get_line`World :: !*World -> (![Char],!*World)
get_line`World w
	# (c,w) = stdin_read_char_world w
	| c>=0
		# c=toEnum c
		| c=='\n'
			= ([],w)
			# (s,w) = get_rest_of_line_world w
			= (c:s,w)
		= io_error_world EOFError w 
  where
  	get_rest_of_line_world :: !*World -> (![Char],!*World)
	get_rest_of_line_world w
		# (c,w) = stdin_read_char_world w
		| c>=0
			# c=toEnum c
			| c=='\n'
				= ([],w)
				# (s,w) = get_rest_of_line_world w
				= (c:s,w)
			= ([],w)

getContents :: IO String
getContents
	= return get_contents
  where
	get_contents
		# c = stdin_read_char
		| c>=0
			#! c=toEnum c
			= c:get_contents
			= []

interact :: (String -> String) -> IO ()
interact f
	= do
		s <- getContents
		putStr (f s)

readFile :: FilePath -> IO String
readFile file_name_cl
	= IO (read_file_world file_name_cl)
where
	read_file_world :: ![Char] -> !*World -> ([Char],!*World)
	read_file_world file_name_cl world
		# (ok,file,world) = sfopen (char_list_to_string file_name_cl) FReadText world
		| not ok
			= io_error_world DoesNotExistError world // or PermissionError
			= (read_chars file,world)

	read_chars file
		# (ok,c,file) = sfreadc file
		| ok
			= c:read_chars file
			= []

writeFile :: FilePath -> String -> IO ()
writeFile file_name_cl cl
	= IO (write_file_world file_name_cl cl)
  where
  	write_file_world :: ![Char] -> ![Char] -> !*World -> (!(),!*World)
  	write_file_world file_name_cl cl world
		# (ok,file,world) = fopen (char_list_to_string file_name_cl) FWriteText world
		| not ok
			= io_error_world PermissionError world
		= close_file_after_writing_world (write_chars cl file) world

appendFile :: FilePath -> String -> IO ()
appendFile file_name_cl cl
	= IO (append_file_world file_name_cl cl)
  where
  	append_file_world :: ![Char] -> ![Char] -> !*World -> (!(),!*World)
  	append_file_world file_name_cl cl world
		# (ok,file,world) = fopen (char_list_to_string file_name_cl) FAppendText world
		| not ok
			= io_error_world PermissionError world
		= close_file_after_writing_world (write_chars cl file) world

write_chars (c:cs) file
	= write_chars cs (fwritec c file)
write_chars [] file
	= file

close_file_after_writing_world :: !*File -> !*World -> (!(),!*World)
close_file_after_writing_world file world
	#  (ok,world) = fclose file world
	| not ok
		= io_error_world FullError world 
		= ((),world)

//readIO :: Read a => String -> IO a
readIO s :== IO (\w -> readIO_exception_if_error`World (readsPrec `0 s) w)

readIO_exception_if_error`World :: ![(a,[.Char])] -> !*World -> (a,!.World)
readIO_exception_if_error`World reads_s w
	= case [x | (x,t) <- reads_s, ("","") <- lex t] of
		[x] -> (x,w)
		[] -> io_error_world (userError "Prelude.readIO: no parse") w
		_ -> io_error_world (userError "Prelude.readIO: ambiguous parse") w

//readLn :: Read a => IO a
readLn :== IO (\w -> readLn_f`World (readsPrec `0) w)

readLn_f`World :: !.([Char] -> [(a,[Char])]) -> !*World -> (a,!*World)
readLn_f`World read_f w
	# (l,w) = get_line`World w
	= readIO_exception_if_error`World (read_f l) w
