implementation module PreludeShow where

from StdString import class toString (..),instance toString Int,instance toString Double
from StdClass import qualified class toReal(..)
from StdFloat import instance toReal Float
from PreludeInternal import showLitChar,string_to_char_list,string_to_char_list_append

ShowChar c s :== c:s

ShowString s1 s2 :== s1++s2

showParen :: !Bool -> ShowS -> ShowS
showParen b p
	= if b then (\t -> '(': (\t -> p (')' : t)) t) else p

instance Show Int where
	show :: !Int -> [Char]
	show i = string_to_char_list (toString i)
	showsPrec _ i = string_to_char_list_append (toString i)

instance Show Double where
	show :: !Double -> [Char]
	show r = string_to_char_list (toString r)
	showsPrec _ r = string_to_char_list_append (toString r)

instance Show Float where
	show :: !Float -> [Char]
	show f = string_to_char_list (toString (StdClass.toReal f))
	showsPrec _ f = string_to_char_list_append (toString (StdClass.toReal f))

instance Show Char where
	show :: !Char -> [Char]
	show '\'' = "'\\''"
	show c = ShowChar '\'' $ showLitChar c $ ['\'']

	showsPrec :: Int -> !Char -> ([Char]->[Char])
	showsPrec p '\'' = ShowString "'\\''"
	showsPrec p c = \t -> ShowChar '\'' $ showLitChar c $ ShowChar '\'' t

	showList :: [Char] -> ([Char]->[Char])
	showList cs = \t -> ShowChar '"' (showl cs t)
	where
		showl "" t = ShowChar '"' t
		showl ('"':cs) t = ShowString "\\\"" (showl cs t)
		showl (c:cs) t = showLitChar c (showl cs t)

instance Show Bool where
	show :: !Bool -> [Char]
	show True = "True"
	show False = "False"
	showsPrec _ i = \ t -> if i ['True';t] ['False';t]

instance Show () where
	show :: !() -> [Char]
	show () = "()"
	showsPrec _ () = \t -> ['()';t]

instance (Show a) =>
  Show [a] where
	show s = showList s ""
	showsPrec p x = showList x

instance (Show a) =>
  Show (Maybe a) where
	showsPrec p (Just v)
		= \s -> ['(Just ' ; shows v (')':s)]
	showsPrec p Nothing
		= \s -> ['Nothing';s]

instance (Show a,Show b) =>
  Show (Either a b) where
	showsPrec p (Left v)
		= \s -> ['(Left ' ; shows v (')':s)]
	showsPrec p (Right v)
		= \s -> ['(Right ' ; shows v (')':s)]

instance Show Ordering where
	showsPrec p EQ
		= \s -> ['EQ';s]
	showsPrec p LT
		= \s -> ['LT';s]
	showsPrec p GT
		= \s -> ['GT';s]

instance (Show e0,Show e1) =>
  Show (e0,e1) where
	show (x,y)
		= '(' : (shows x
			$ ',' : shows y
			")")
	showsPrec p (x,y)
		= \t -> '(' : (shows x
			$ ',' : shows y
			[')';t])

instance (Show e0,Show e1,Show e2) =>
  Show (e0,e1,e2) where
	show (e0,e1,e2)
		= '(' : (shows e0 
			$ ',' : (shows e1
			$ ',' : shows e2
			")"))
	showsPrec p (e0,e1,e2)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : shows e2
			[')';t]))

instance (Show e0,Show e1,Show e2,Show e3) =>
  Show (e0,e1,e2,e3) where
	show (e0,e1,e2,e3)
		= '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : shows e3
			")")))
	showsPrec p (e0,e1,e2,e3)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : shows e3
			[')';t])))

instance (Show e0,Show e1,Show e2,Show e3,Show e4) =>
  Show (e0,e1,e2,e3,e4) where
	show (e0,e1,e2,e3,e4)
		= '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : shows e4
			")"))))
	showsPrec p (e0,e1,e2,e3,e4)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : shows e4
			[')';t]))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5) =>
  Show (e0,e1,e2,e3,e4,e5) where
	show (e0,e1,e2,e3,e4,e5)
		= '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : shows e5
			")")))))
	showsPrec p (e0,e1,e2,e3,e4,e5)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : shows e5
			[')';t])))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6) =>
  Show (e0,e1,e2,e3,e4,e5,e6) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : shows e6
			[')';t]))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : shows e7
			[')';t])))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7,Show e8) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7,e8) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7,e8)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : (shows e7
			$ ',' : shows e8
			[')';t]))))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7,Show e8,Show e9) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : (shows e7
			$ ',' : (shows e8
			$ ',' : shows e9
			[')';t])))))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7,Show e8,Show e9,Show e10) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : (shows e7
			$ ',' : (shows e8
			$ ',' : (shows e9
			$ ',' : shows e10
			[')';t]))))))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7,Show e8,Show e9,Show e10,Show e11) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : (shows e7
			$ ',' : (shows e8
			$ ',' : (shows e9
			$ ',' : (shows e10
			$ ',' : shows e11
			[')';t])))))))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7,Show e8,Show e9,Show e10,Show e11,Show e12) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : (shows e7
			$ ',' : (shows e8
			$ ',' : (shows e9
			$ ',' : (shows e10
			$ ',' : (shows e11
			$ ',' : shows e12
			[')';t]))))))))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7,Show e8,Show e9,Show e10,Show e11,Show e12,Show e13) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : (shows e7
			$ ',' : (shows e8
			$ ',' : (shows e9
			$ ',' : (shows e10
			$ ',' : (shows e11
			$ ',' : (shows e12
			$ ',' : shows e13
			[')';t])))))))))))))

instance (Show e0,Show e1,Show e2,Show e3,Show e4,Show e5,Show e6,Show e7,Show e8,Show e9,Show e10,Show e11,Show e12,Show e13,Show e14) =>
  Show (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13,e14) where
	showsPrec p (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13,e14)
		= \t -> '(' : (shows e0
			$ ',' : (shows e1
			$ ',' : (shows e2
			$ ',' : (shows e3
			$ ',' : (shows e4
			$ ',' : (shows e5
			$ ',' : (shows e6
			$ ',' : (shows e7
			$ ',' : (shows e8
			$ ',' : (shows e9
			$ ',' : (shows e10
			$ ',' : (shows e11
			$ ',' : (shows e12
			$ ',' : (shows e13
			$ ',' : shows e14
			[')';t]))))))))))))))
