module Time (
	Month(..),Day(..),CalendarTime(..),TimeDiff(..),ClockTime,
	getClockTime,toCalendarTime,
	localCalendarTimeToClockTime`,
	toUTCTime,toClockTime,
	addToClockTime,diffClockTimes,
	calendarTimeToString,formatCalendarTime
	) where

--import Locale (TimeLocale(..),defaultTimeLocale) -- bug
import Locale (TimeLocale(wDays,months,dateTimeFmt,amPm,time12Fmt,timeFmt,dateFmt),defaultTimeLocale)
import StdInt (bitand)
import qualified StdInt ((>>),(<<))

from PreludeInternal import string_to_char_list

data Month
	= January | February | March | April
	| May | June | July | August
	| September | October | November | December
	deriving (Eq, Ord, Enum, Bounded, Ix, Read, Show)

data Day
	= Sunday | Monday | Tuesday | Wednesday | Thursday
	| Friday | Saturday
	deriving (Eq, Ord, Enum, Bounded, Ix, Read, Show)

data CalendarTime
	= CalendarTime {
		ctYear :: Int,
		ctMonth :: Month,
		ctDay, ctHour, ctMin, ctSec :: Int,
		ctPicosec :: Integer,
		ctWDay :: Day,
		ctYDay :: Int,
		ctTZName :: String,
		ctTZ :: Int,
		ctIsDST :: Bool
	} deriving (Eq, Ord, Read, Show)

data TimeDiff
	= TimeDiff {
		tdYear, tdMonth, tdDay, tdHour, tdMin, tdSec :: Int,
		tdPicosec :: Integer
	} deriving (Eq, Ord, Read, Show)

-- newtype ClockTime = ClockTime {#Int} -- 3

:: ClockTime = ! {ct_milli_sec_l::!Int, ct_milli_sec_h::!Int, ct_pico_sec::!Int}

instance Eq ClockTime where
	t1 == t2 = t1~ct_milli_sec_h==t2~ct_milli_sec_h && t1~ct_milli_sec_l==t2~ct_milli_sec_l && t1~ct_pico_sec==t2~ct_pico_sec
	t1 /= t2 = t1~ct_milli_sec_h/=t2~ct_milli_sec_h || t1~ct_milli_sec_l/=t2~ct_milli_sec_l || t1~ct_pico_sec==t2~ct_pico_sec

instance Ord ClockTime where
	t1 < t2
		| t1~ct_milli_sec_h/=t2~ct_milli_sec_h
			= ltU t1~ct_milli_sec_h t2~ct_milli_sec_h
		| t1~ct_milli_sec_l/=t2~ct_milli_sec_l
			= ltU t1~ct_milli_sec_l t2~ct_milli_sec_l
			= ltU t1~ct_pico_sec t2~ct_pico_sec
	t1 >= t2
		| t1~ct_milli_sec_h/=t2~ct_milli_sec_h
			= not (ltU t1~ct_milli_sec_h t2~ct_milli_sec_h)
		| t1~ct_milli_sec_l/=t2~ct_milli_sec_l
			= not (ltU t1~ct_milli_sec_l t2~ct_milli_sec_l)
			= not (ltU t1~ct_pico_sec t2~ct_pico_sec)
	t1 > t2
		| t1~ct_milli_sec_h/=t2~ct_milli_sec_h
			= ltU t2~ct_milli_sec_h t1~ct_milli_sec_h
		| t1~ct_milli_sec_l/=t2~ct_milli_sec_l
			= ltU t2~ct_milli_sec_l t1~ct_milli_sec_l
			= ltU t2~ct_pico_sec t1~ct_pico_sec
	t1 <= t2
		| t1~ct_milli_sec_h/=t2~ct_milli_sec_h
			= not (ltU t2~ct_milli_sec_h t1~ct_milli_sec_h)
		| t1~ct_milli_sec_l/=t2~ct_milli_sec_l
			= not (ltU t2~ct_milli_sec_l t1~ct_milli_sec_l)
			= not (ltU t2~ct_pico_sec t1~ct_pico_sec)

mulud :: !Int -> !Int -> (!Int,!Int);
mulud a b = `code inline {
	mulUUL
}

addLU :: !Int -> !Int -> !Int -> (!Int,!Int);
addLU _ _ _ = `code inline {
	addLU
}

subLU :: !Int -> !Int -> !Int -> (!Int,!Int);
subLU _ _ _ = `code inline {
	subLU
}

ushiftr :: !Int -> !Int -> Int;
ushiftr a b = `code inline {
	shiftrU
}

divdu :: !Int -> !Int -> !Int -> (!Int,!Int)
divdu a b c = `code inline {
	divLU
}

ltU :: !Int -> !Int -> Bool;
ltU a b = `code inline {
	ltU
}

type FILETIME = {#Int} -- 2
newtype SYSTEMTIME = SYSTEMTIME {#Int} -- 4

SYSTEMTIME_year_and_month_index :== `0
SYSTEMTIME_day_of_week_and_day_index :== `1
SYSTEMTIME_hour_and_minute_index :== `2
SYSTEMTIME_second_and_milliseconds_index :== `3

type TIME_ZONE_INFORMATION = {#Int}

TIME_ZONE_ID_INVALID :== `-1
TIME_ZONE_ID_UNKNOWN :== `0

type LP_TIME_ZONE_INFORMATION_Int = Int

GetSystemTimeAsFileTime :: !FILETIME -> FILETIME
GetSystemTimeAsFileTime a
	= `code {
		push_a 0
		ccall GetSystemTimeAsFileTime@4 "PA:V:A"
	}

FileTimeToSystemTime :: !FILETIME -> !SYSTEMTIME -> (!Int,!SYSTEMTIME)
FileTimeToSystemTime f s
	= `code {
		push_a 0
		update_a 2 1
		ccall FileTimeToSystemTime@8 "PAA:I:A"
	}

SystemTimeToTzSpecificLocalTime :: !LP_TIME_ZONE_INFORMATION_Int -> !SYSTEMTIME -> !SYSTEMTIME -> (!Int,!SYSTEMTIME)
SystemTimeToTzSpecificLocalTime p_time_zone universal_time local_time
	= `code {
		push_a 0
		update_a 2 1
		ccall SystemTimeToTzSpecificLocalTime@12 "PpAA:I:A"
	}

SystemTimeToTzSpecificLocalTime_a :: !TIME_ZONE_INFORMATION -> !SYSTEMTIME -> !SYSTEMTIME -> (!Int,!SYSTEMTIME)
SystemTimeToTzSpecificLocalTime_a time_zone universal_time local_time
	= `code {
		push_a 0
		update_a 2 1
		update_a 3 2
		ccall SystemTimeToTzSpecificLocalTime@12 "PAAA:I:A"
	}

SystemTimeToFileTime :: !SYSTEMTIME -> !*FILETIME -> (!Int,!*FILETIME)
SystemTimeToFileTime s f
	= `code {
		push_a 0
		update_a 2 1
		ccall SystemTimeToFileTime@8 "PAA:I:A"
	}

LocalFileTimeToFileTime :: !FILETIME -> !FILETIME -> (!Int,!FILETIME)
LocalFileTimeToFileTime local_file_time file_time
	= `code {
		push_a 0
		update_a 2 1
		ccall LocalFileTimeToFileTime@8 "PAA:I:A"
	}

GetTimeZoneInformation :: !TIME_ZONE_INFORMATION -> (!Int,!TIME_ZONE_INFORMATION)
GetTimeZoneInformation tzi
	= `code {
		push_a 0
		ccall GetTimeZoneInformation@4 "PA:I:A"
	}

getClockTime :: IO ClockTime
getClockTime = IO getClockTimeWorld

getClockTimeWorld :: !*World -> (!ClockTime,!*World)
getClockTimeWorld w
	= (unsafeGetClockTime,w)

unsafeGetClockTime :: ClockTime
unsafeGetClockTime
	# file_time = GetSystemTimeAsFileTime {0,0}
	  file_time_l = file_time~[0]
	  file_time_h = file_time~[1]
	  (rh,qh) = divdu 0 file_time_h 10000
	  (rl,ql) = divdu rh file_time_l 10000
--	= ClockTime {ql,qh,rl*100000}
	= {ct_milli_sec_l=ql,ct_milli_sec_h=qh,ct_pico_sec=rl*100000}

clock_time_to_file_time_and_pico_seconds :: !ClockTime -> (!FILETIME,!Int)
{-
clock_time_to_file_time_and_pico_seconds (ClockTime clock_time)
	# cl=clock_time~[0]
	  ch=clock_time~[1]
	  cp=clock_time~[2]
	  (fh,fl) = mulud cl 10000
	  fh = fh + 10000*ch
	= ({fl,fh},cp)
-}
clock_time_to_file_time_and_pico_seconds {ct_milli_sec_l,ct_milli_sec_h,ct_pico_sec}
	# (fh,fl) = mulud ct_milli_sec_l 10000
	  fh = fh + 10000*ct_milli_sec_h
	= ({fl,fh},ct_pico_sec)

file_time_and_pico_seconds_to_clock_time :: !FILETIME -> !Int -> ClockTime
file_time_and_pico_seconds_to_clock_time file_time pico_seconds
	# file_time_l = file_time~[0]
	  file_time_h = file_time~[1]
 	  (rh,qh) = divdu 0 file_time_h 10000
	  (rl,ql) = divdu rh file_time_l 10000 -- rl == 0
--	= ClockTime {ql,qh,pico_seconds}
	= {ct_milli_sec_l=ql,ct_milli_sec_h=qh,ct_pico_sec=pico_seconds}

file_time_hl_and_pico_seconds_to_clock_time :: !Int -> !Int -> !Int -> ClockTime
file_time_hl_and_pico_seconds_to_clock_time file_time_h file_time_l pico_seconds
 	# (rh,qh) = divdu 0 file_time_h 10000
	  (rl,ql) = divdu rh file_time_l 10000 -- rl == 0
--	= ClockTime {ql,qh,pico_seconds}
	= {ct_milli_sec_l=ql,ct_milli_sec_h=qh,ct_pico_sec=pico_seconds}

toCalendarTime :: !ClockTime -> IO CalendarTime
toCalendarTime clock_time = IO (toCalendarTimeWorld clock_time)

toCalendarTimeWorld :: !ClockTime -> !*World -> (!CalendarTime,!*World)
toCalendarTimeWorld clock_time w = (unsafeToCalendarTime clock_time,w)

unsafeToCalendarTime :: !ClockTime -> CalendarTime
unsafeToCalendarTime clock_time
	# (file_time,pico_seconds) = clock_time_to_file_time_and_pico_seconds clock_time
	#!(r0,system_time@(SYSTEMTIME system_time_a)) = FileTimeToSystemTime file_time (SYSTEMTIME (createArray `4 `0))
	  (r1,SYSTEMTIME local_time_a) = SystemTimeToTzSpecificLocalTime `0 system_time (SYSTEMTIME (createArray `4 `0))
	  (r2,tzi) = GetTimeZoneInformation (createArray `43 `0)
	# year_and_month = local_time_a~[SYSTEMTIME_year_and_month_index]
	  day_of_week_and_day = local_time_a~[SYSTEMTIME_day_of_week_and_day_index]
	  hour_and_minute = local_time_a~[SYSTEMTIME_hour_and_minute_index]
	  second_and_milliseconds = local_time_a~[SYSTEMTIME_second_and_milliseconds_index]
	#! year = year_and_month bitand `0xffff
	   month = (year_and_month StdInt.>> 16) - `1
	   day_of_week = day_of_week_and_day bitand `0xffff
	   day = day_of_week_and_day StdInt.>> 16
	   hour = hour_and_minute bitand `0xffff
	   minute = hour_and_minute StdInt.>> 16
	   second = second_and_milliseconds bitand `0xffff
	   milliseconds = second_and_milliseconds StdInt.>> 16
	# (time_zone_name,dst) = compute_time_zone_name_and_dst r2 tzi system_time local_time_a
	= CalendarTime {
		ctYear = year,
		ctMonth = toEnum month,
		ctDay = day,
		ctHour = hour,
		ctMin = minute,
		ctSec = second,
		ctPicosec = toInteger milliseconds * 1000000000 + toInteger pico_seconds,
		ctWDay = toEnum day_of_week,
		ctYDay = day_of_year year month day,
		ctTZName = time_zone_name,
		ctTZ = compute_time_zone_bias local_time_a system_time_a,
		ctIsDST = dst
	}
  where
	compute_time_zone_bias local_time utc_time
		# utc_year_and_month = utc_time~[SYSTEMTIME_year_and_month_index]
		  utc_day_of_week_and_day = utc_time~[SYSTEMTIME_day_of_week_and_day_index]
		  utc_hour_and_minute = utc_time~[SYSTEMTIME_hour_and_minute_index]
		  utc_year = utc_year_and_month bitand `0xffff
		  utc_month = utc_year_and_month StdInt.>> 16
		  utc_day = utc_day_of_week_and_day StdInt.>> 16
		  utc_hour = utc_hour_and_minute bitand `0xffff
		  utc_minute = utc_hour_and_minute StdInt.>> 16
	
		# local_year_and_month = local_time~[SYSTEMTIME_year_and_month_index]
		  local_day_of_week_and_day = local_time~[SYSTEMTIME_day_of_week_and_day_index]
		  local_hour_and_minute = local_time~[SYSTEMTIME_hour_and_minute_index]
		  local_year = local_year_and_month bitand `0xffff
		  local_month = local_year_and_month StdInt.>> 16
		  local_day = local_day_of_week_and_day StdInt.>> 16
		  local_hour = local_hour_and_minute bitand `0xffff
		  local_minute = local_hour_and_minute StdInt.>> 16

		#! hour_diff = local_hour-utc_hour
		   minute_diff = (local_minute-utc_minute)*`60
		| local_year==utc_year && local_month==utc_month && local_day==utc_day
			= hour_diff*`3600+minute_diff
		| local_year>utc_year || (local_year==utc_year && local_month*32+local_day>=utc_month*32+utc_day)
			= (hour_diff+24)*`3600+minute_diff
			= (hour_diff-24)*`3600+minute_diff

  	compute_time_zone_name_and_dst :: Int -> {#Int} -> SYSTEMTIME -> {#Int} -> (![Char],!Bool)
	compute_time_zone_name_and_dst r2 tzi system_time local_time_a
  		| r2==TIME_ZONE_ID_INVALID
			= ("",False)
		| r2==TIME_ZONE_ID_UNKNOWN
			= time_zone_name_and_no_dst tzi
			# no_dst_tzi = {{e;;e<-:tzi} & [17]=0,[38]=0}
			# (r3,SYSTEMTIME no_dst_local_time_a) = SystemTimeToTzSpecificLocalTime_a no_dst_tzi system_time (SYSTEMTIME (createArray `4 `0))
			| r3/=TIME_ZONE_ID_INVALID
				| same_day_hour_and_minute local_time_a no_dst_local_time_a
					= time_zone_name_and_no_dst tzi
					= (string_to_char_list (uzstring_from_int_array tzi 44 32),True)
				= time_zone_name_and_no_dst tzi
	  where
		time_zone_name_and_no_dst tzi
			= (string_to_char_list (uzstring_from_int_array tzi 2 32),False)

toUTCTime :: !ClockTime -> CalendarTime
toUTCTime clock_time
	# (file_time,pico_seconds) = clock_time_to_file_time_and_pico_seconds clock_time
	  (r0,SYSTEMTIME system_time) = FileTimeToSystemTime file_time (SYSTEMTIME (createArray `4 `0))
	  year_and_month = system_time~[SYSTEMTIME_year_and_month_index]
	  day_of_week_and_day = system_time~[SYSTEMTIME_day_of_week_and_day_index]
	  hour_and_minute = system_time~[SYSTEMTIME_hour_and_minute_index]
	  second_and_milliseconds = system_time~[SYSTEMTIME_second_and_milliseconds_index]
	#! year = year_and_month bitand `0xffff
	   month = (year_and_month StdInt.>> 16) - `1
	   day_of_week = day_of_week_and_day bitand `0xffff
	   day = day_of_week_and_day StdInt.>> 16
	   hour = hour_and_minute bitand `0xffff
	   minute = hour_and_minute StdInt.>> 16
	   second = second_and_milliseconds bitand `0xffff
	   milliseconds = second_and_milliseconds StdInt.>> 16
	= CalendarTime {
		ctYear = year,
		ctMonth = toEnum month,
		ctDay = day,
		ctHour = hour,
		ctMin = minute,
		ctSec = second, 
		ctPicosec = toInteger milliseconds * 1000000000 + toInteger pico_seconds,
		ctWDay = toEnum day_of_week,
		ctYDay = day_of_year year month day,
		ctTZName = "UTC",
		ctTZ = 0,
		ctIsDST = False
	}

toClockTime :: !CalendarTime -> ClockTime
toClockTime CalendarTime {ctYear,ctMonth,ctDay,ctHour,ctMin,ctSec,ctPicosec {-,ctTZ-} }
	# (milli_seconds,pico_seconds) = quotRem ctPicosec 1000000000
	  year_and_month = (ctYear bitand `0xffff) + (((fromEnum ctMonth)+1) StdInt.<< 16)
	  day_of_week_and_day = ctDay StdInt.<< 16
	  hour_and_minute = (ctHour bitand `0xffff) + (ctMin StdInt.<< 16)
	  second_and_milliseconds = (ctSec bitand `0xffff) + (fromInteger milli_seconds StdInt.<< 16)
	  system_time_a = {year_and_month,day_of_week_and_day,hour_and_minute,second_and_milliseconds}
	  (r0,file_time) = SystemTimeToFileTime (SYSTEMTIME system_time_a) (createArray 2 0)
	= file_time_and_pico_seconds_to_clock_time file_time (fromInteger pico_seconds)

localCalendarTimeToClockTime` :: !CalendarTime -> IO ClockTime
localCalendarTimeToClockTime` calendar_time = IO (localCalendarTimeToClockTimeWorld calendar_time)

localCalendarTimeToClockTimeWorld ::  !CalendarTime -> !*World -> (!ClockTime,!*World)
localCalendarTimeToClockTimeWorld calendar_time world
	= (unsafeLocalCalendarTimeToClockTime calendar_time,world)

unsafeLocalCalendarTimeToClockTime :: !CalendarTime -> ClockTime
unsafeLocalCalendarTimeToClockTime CalendarTime {ctYear,ctMonth,ctDay,ctHour,ctMin,ctSec,ctPicosec {-,ctTZ-} }
	# (milli_seconds,pico_seconds) = quotRem ctPicosec 1000000000
	  year_and_month = (ctYear bitand `0xffff) + (((fromEnum ctMonth)+1) StdInt.<< 16)
	  day_of_week_and_day = ctDay StdInt.<< 16
	  hour_and_minute = (ctHour bitand `0xffff) + (ctMin StdInt.<< 16)
	  second_and_milliseconds = (ctSec bitand `0xffff) + (fromInteger milli_seconds StdInt.<< 16)
	  system_time_a = {year_and_month,day_of_week_and_day,hour_and_minute,second_and_milliseconds}
	  (r0,file_time) = SystemTimeToFileTime (SYSTEMTIME system_time_a) (createArray 2 0)
	  (r1,utc_file_time) = LocalFileTimeToFileTime file_time (createArray 2 0)
	  (r2,utc_system_time) = FileTimeToSystemTime utc_file_time (SYSTEMTIME (createArray `4 `0))
	  (r3,SYSTEMTIME local_time_a) = SystemTimeToTzSpecificLocalTime `0 utc_system_time (SYSTEMTIME (createArray `4 `0))
	| same_day_hour_and_minute system_time_a local_time_a
		= file_time_and_pico_seconds_to_clock_time utc_file_time (fromInteger pico_seconds)
	 	# (r4,local_file_time2) = SystemTimeToFileTime (SYSTEMTIME local_time_a) (createArray 2 0)
		  local_file_time2_l = local_file_time2~[0]
		  local_file_time2_h = local_file_time2~[1]
		  file_time_l = file_time~[0]
		  file_time_h = file_time~[1]
		  (h,l ) = subLU (file_time_h-local_file_time2_h) file_time_l local_file_time2_l
		  utc_file_time_l = utc_file_time~[0]
		  utc_file_time_h = utc_file_time~[1]
		  (utc_file_time_h,utc_file_time_l) = addLU (utc_file_time_h+h) utc_file_time_l l
		= file_time_hl_and_pico_seconds_to_clock_time utc_file_time_h utc_file_time_l (fromInteger pico_seconds)

addToClockTime :: !TimeDiff -> !ClockTime -> ClockTime
{-
addToClockTime td @ (TimeDiff {tdYear,tdMonth,tdDay,tdHour,tdMin,tdSec,tdPicosec}) (ClockTime clock_time)
	# milli_sec_l = clock_time~[0]
	  milli_sec_h = clock_time~[1]
	  pico_sec = clock_time~[2]
-}
addToClockTime td @ (TimeDiff {tdYear,tdMonth,tdDay,tdHour,tdMin,tdSec,tdPicosec})
	{ct_milli_sec_l=milli_sec_l,ct_milli_sec_h=milli_sec_h,ct_pico_sec=pico_sec}
	# (milli_sec_h,milli_sec_l,pico_sec) = add_pico_sec_to_milli_sec td milli_sec_h milli_sec_l pico_sec
	  (milli_sec_h,milli_sec_l) = add_sec_to_milli_sec td milli_sec_h milli_sec_l
	  (milli_sec_h,milli_sec_l) = add_min_to_milli_sec td milli_sec_h milli_sec_l
	  (milli_sec_h,milli_sec_l) = add_hour_to_milli_sec td milli_sec_h milli_sec_l
	  (milli_sec_h,milli_sec_l) = add_day_to_milli_sec td milli_sec_h milli_sec_l
--	  clock_time = ClockTime {milli_sec_l,milli_sec_h,pico_sec}
	  clock_time = {ct_milli_sec_l=milli_sec_l,ct_milli_sec_h=milli_sec_h,ct_pico_sec=pico_sec}
	| tdYear==0 && tdMonth==0
		= clock_time
		# calendar_time@(CalendarTime {ctYear,ctMonth}) = toUTCTime clock_time
		  month_n = fromEnum ctMonth
		  (d_year,month_n) = divMod (month_n+tdMonth) 12
		  ctYear = ctYear+tdYear+d_year
		= toClockTime calendar_time {ctYear=ctYear,ctMonth=toEnum month_n}
	where
	 	add_pico_sec_to_milli_sec TimeDiff {tdPicosec} milli_sec_h milli_sec_l pico_sec
	 		| tdPicosec==0
		  		= (milli_sec_h,milli_sec_l,pico_sec)
		  	| tdPicosec>0
				# (milli_seconds,pico_seconds) = quotRem tdPicosec 1000000000
				# pico_sec = pico_sec+fromInteger pico_seconds
				| pico_sec < 1000000000
					# (milli_sec_h,milli_sec_l) = addLU milli_sec_h milli_sec_l (fromInteger milli_seconds)
					= (milli_sec_h,milli_sec_l,pico_sec)
					# (milli_sec_h,milli_sec_l) = addLU milli_sec_h milli_sec_l (1+fromInteger milli_seconds)
					= (milli_sec_h,milli_sec_l,pico_sec-1000000000)
				# (milli_seconds,pico_seconds) = quotRem (-tdPicosec) 1000000000
				# pico_sec = pico_sec-fromInteger pico_seconds
				| pico_sec >= 0
					# (milli_sec_h,milli_sec_l) = subLU milli_sec_h milli_sec_l (fromInteger milli_seconds)
					= (milli_sec_h,milli_sec_l,pico_sec)
					# (milli_sec_h,milli_sec_l) = subLU milli_sec_h milli_sec_l (1+fromInteger milli_seconds)
					= (milli_sec_h,milli_sec_l,pico_sec+1000000000)

		add_sec_to_milli_sec TimeDiff {tdSec} milli_sec_h milli_sec_l
			| tdSec==0
				= (milli_sec_h,milli_sec_l)
			| tdSec>0
				# (h,l) = mulud tdSec 1000
				= addLU (milli_sec_h+h) milli_sec_l l
				# (h,l) = mulud (-tdSec) 1000
				= subLU (milli_sec_h-h) milli_sec_l l

		add_min_to_milli_sec TimeDiff {tdMin} milli_sec_h milli_sec_l
			| tdMin==0
				= (milli_sec_h,milli_sec_l)
			| tdMin>0
				# (h,l) = mulud tdMin 60000
				= addLU (milli_sec_h+h) milli_sec_l l
				# (h,l) = mulud (-tdMin) 60000
				= subLU (milli_sec_h-h) milli_sec_l l

		add_hour_to_milli_sec TimeDiff {tdHour} milli_sec_h milli_sec_l
			| tdHour==0
				= (milli_sec_h,milli_sec_l)
			| tdHour>0
				# (h,l) = mulud tdHour 3600000
				= addLU (milli_sec_h+h) milli_sec_l l
				# (h,l) = mulud (-tdHour) 3600000
				= subLU (milli_sec_h-h) milli_sec_l l

		add_day_to_milli_sec TimeDiff {tdDay} milli_sec_h milli_sec_l
			| tdDay==0
				= (milli_sec_h,milli_sec_l)
			| tdDay>0
				# (h,l) = mulud tdDay 86400000
				= addLU (milli_sec_h+h) milli_sec_l l
				# (h,l) = mulud (-tdDay) 86400000
				= subLU (milli_sec_h-h) milli_sec_l l

diffClockTimes :: !ClockTime -> !ClockTime -> TimeDiff
diffClockTimes clock_time1 clock_time2
	#  (year1,month1,day1,hour1,minute1,second1,pico_sec1) = toTimeDiffT clock_time1
	   (year2,month2,day2,hour2,minute2,second2,pico_sec2) = toTimeDiffT clock_time2
	#! year=year1-year2
	   month=month1-month2
	   day=day1-day2
	   hour=hour1-hour2
	   minute=minute1-minute2
	   second=second1-second2
	#  pico_sec = pico_sec1-pico_sec2
	= TimeDiff {tdYear=year,tdMonth=month,tdDay=day,tdHour=hour,tdMin=minute,tdSec=second,tdPicosec=pico_sec}
  where
	toTimeDiffT clock_time
		# (file_time,pico_seconds) = clock_time_to_file_time_and_pico_seconds clock_time
		# (r0,SYSTEMTIME system_time) = FileTimeToSystemTime file_time (SYSTEMTIME (createArray `4 `0))
		  year_and_month = system_time~[SYSTEMTIME_year_and_month_index]
		  day_of_week_and_day = system_time~[SYSTEMTIME_day_of_week_and_day_index]
		  hour_and_minute = system_time~[SYSTEMTIME_hour_and_minute_index]
		  second_and_milliseconds = system_time~[SYSTEMTIME_second_and_milliseconds_index]
		#! year = year_and_month bitand `0xffff
		   month = year_and_month StdInt.>> 16
		   day = day_of_week_and_day StdInt.>> 16
		   hour = hour_and_minute bitand `0xffff
		   minute = hour_and_minute StdInt.>> 16
		   second = second_and_milliseconds bitand `0xffff
		   milliseconds = second_and_milliseconds StdInt.>> 16
		= (year,month,day,hour,minute,second,toInteger milliseconds * 1000000000 + toInteger pico_seconds)

is_no_leap_year year :== year `bitand` 3/=0 || not (year `rem` 100/=0 && year `rem` 400/=0)

day_of_year year month day
	#! day_of_non_leap_year = first_day_in_month_to_day_of_year~[month] + (day-`1)
	| month<=1 || is_no_leap_year year
		= day_of_non_leap_year
		= day_of_non_leap_year+`1

first_day_in_month_to_day_of_year :: {#Int}
first_day_in_month_to_day_of_year
	=: { 0,31,59,90,120,151,181,212,244,274,305,335}
--	   {31,28,31,30, 31, 30, 31, 31, 30, 31, 30 ,31}

uzstring_from_int_array :: {#Int} -> Int -> Int -> {#Char}
uzstring_from_int_array a i0 n
	# n = find_zero 0 i0 n a
	= {toEnum (a~[i StdInt.>> 1] StdInt.>> ((i bitand 1) StdInt.<< 4))::Char ;; i<-[i0..i0+n-1]}
  where
	find_zero s i0 n a
		| s<n
			# i=i0+s
			| (a~[i StdInt.>> 1] StdInt.>> ((i bitand 1) StdInt.<< 4)) bitand 0xffff/=0
				= find_zero (s+1) i0 n a
				= s
			= n

same_day_hour_and_minute :: !{#Int} -> !{#Int} -> Bool
same_day_hour_and_minute time1 time2
	# dst_day = time1~[SYSTEMTIME_day_of_week_and_day_index] StdInt.>> 16
	  dst_hour_and_minute = time1~[SYSTEMTIME_hour_and_minute_index]
 	  no_dst_day = time2~[SYSTEMTIME_day_of_week_and_day_index] StdInt.>> 16
	  no_dst_hour_and_minute = time2~[SYSTEMTIME_hour_and_minute_index]
	= dst_day==no_dst_day && dst_hour_and_minute==no_dst_hour_and_minute

calendarTimeToString :: CalendarTime -> String
calendarTimeToString calendar_time
	= formatCalendarTime defaultTimeLocale "%c" calendar_time

formatCalendarTime :: TimeLocale -> String -> CalendarTime -> String
formatCalendarTime l fmt ct@(CalendarTime year mon day hour min sec sdec wday yday tzname _ _)
	= doFmt fmt
  where
	doFmt ('%':c:cs) = decode c ++ doFmt cs
	doFmt (c:cs) = c : doFmt cs
	doFmt "" = ""

	decode 'A' = fst (wDays l !! fromEnum wday)
	decode 'a' = snd (wDays l !! fromEnum wday)
	decode 'B' = fst (months l !! fromEnum mon)
	decode 'b' = snd (months l !! fromEnum mon)
	decode 'h' = snd (months l !! fromEnum mon)
	decode 'C' = show2 (year `quot` 100)
	decode 'c' = doFmt (dateTimeFmt l)
	decode 'D' = doFmt "%m/%d/%y"
	decode 'd' = show2 day
	decode 'e' = show2' day
	decode 'H' = show2 hour
	decode 'I' = show2 (to12 hour)
	decode 'j' = show3 yday
	decode 'k' = show2' hour
	decode 'l' = show2' (to12 hour)
	decode 'M' = show2 min
	decode 'm' = show2 (fromEnum mon+1)
	decode 'n' = "\n"
	decode 'p' = if hour < 12 then fst (amPm l) else snd (amPm l)
	decode 'R' = doFmt "%H:%M"
	decode 'r' = doFmt (time12Fmt l)
	decode 'T' = doFmt "%H:%M:%S"
	decode 't' = "\t"
	decode 'S' = show2 sec
	decode 's' = show2 sec
	decode 'U' = show2 ((yday + 7 - fromEnum wday) `div` 7)
	decode 'u' = show (let n = fromEnum wday in if n == 0 then 7 else n)
	decode 'V'
		= let
		     (week, days) = (yday + 7 - if fromEnum wday > 0 then fromEnum wday - 1 else 6) `divMod` 7
		  in show2 (if days >= 4 then week+1 else if week == 0 then 53 else week)
	decode 'W' = show2 ((yday + 7 - if fromEnum wday > 0 then fromEnum wday - 1 else 6) `div` 7)
	decode 'X' = doFmt (timeFmt l)
	decode 'x' = doFmt (dateFmt l)
	decode 'Y' = show year
	decode 'y' = show2 (year `rem` 100)
	decode 'Z' = tzname
	decode '%' = "%"
	decode c = [c]

	show2 x = [int_to_digit (x `quot` 10), int_to_digit (x `rem` 10)]
	show2' x = if x < 10 then [' ', int_to_digit x] else show2 x
	show3 x = int_to_digit (x `quot` 100) : show2 (x `rem` 100)

	int_to_digit i = toEnum (48 + i)

	to12 :: Int -> Int
	to12 h
		= let h' = h `mod` 12 in
		  if h' == 0 then 12 else h'
